# -*- coding:utf-8 -*-

"""
Conditionally load models from coop.base_models or from coop_local.models.local_models
to make them globally available from coop_locals.models,
without patching django loading process

http://effbot.org/zone/import-confusion.htm

"""


from django.conf import settings
import logging
from django.core.exceptions import ImproperlyConfigured
import inspect
from django.db.models.base import Model as DjangoModel
from coop_local.models import local_models  # DO NOT REMOVE !!!

log = logging.getLogger('coop')


def import_class(cl):
    d = cl.rfind(".")
    classname = cl[d + 1:len(cl)]
    m = __import__(cl[0:d], globals(), locals(), [classname])
    return getattr(m, classname)


def relocalize_model(model_name, origin, target):
    class_name = str(origin.__name__) + '.' + model_name
    klass = import_class(class_name)
    setattr(target, model_name, klass)
    #log.debug('--> "%s" is now available from <%s>.' % (model_name, target.__name__))


def load_models(base_models, local_models, main_module):
    apps = {'local': [], 'base': [], 'other': []}
    for app in settings.BASE_COOP_LOCAL_MODELS:
        if app[0] in settings.INSTALLED_APPS:
            for model_name in app[1]:
                try:
                    # first, we try to load customized coop Models from local_models (they should have the same name)
                    relocalize_model(model_name, local_models, main_module)
                    apps['local'].append(model_name)
                except Exception, msg:
                    #log.debug('"%s" has not been customized in local_models...' % model_name)
                    # if it's not there, we assume we can load it from the base models
                    try:
                        relocalize_model(model_name, base_models, main_module)
                        apps['base'].append(model_name)

                    except Exception, msg:
                        raise ImproperlyConfigured('Unable to load "%s" from local or base models : %s' % (model_name, msg))

    # Looking for non-coop models also declared in coop_local
    custmodel_list = [(name, obj) for (name, obj) in inspect.getmembers(local_models, inspect.isclass) if (
                        hasattr(obj, '__class__')
                        and DjangoModel in obj.__mro__
                        and 'coop_local' in obj.__module__
                        and name not in apps['local']
                        and name not in apps['base']
                    )]

    for model_name, obj in custmodel_list:
        relocalize_model(model_name, local_models, main_module)
        apps['other'].append(model_name)

    #log.debug('Vanilla coop models installed : %s' % ', '.join([x[0] for x in custmodel_list]))

    log.debug('*** VANILLA COOP MODELS : %s' % ', '.join(apps['base']))
    log.debug('*** CUSTOMIZED COOP MODELS : %s' % ', '.join(apps['local']))
    log.debug('*** ADDITIONAL MODELS : %s' % ', '.join(apps['other']))



# -*- coding:utf-8 -*-

import calendar
import itertools
from datetime import datetime, timedelta, time
from django import http
from django.conf import settings
from django.db import models
from django.template.context import RequestContext
from django.shortcuts import get_object_or_404, render_to_response, redirect
from django.core.urlresolvers import reverse
from coop_local.models import Event, EventCategory, Occurrence, Calendar
from coop.agenda import utils, forms
from coop.agenda.conf import settings as agenda_settings

from dateutil import parser

if settings.FIRST_DAY_OF_WEEK is not None:
    if settings.FIRST_DAY_OF_WEEK == 0:
        calendar.setfirstweekday(6)
    else:
        calendar.setfirstweekday(settings.FIRST_DAY_OF_WEEK - 1)
elif agenda_settings.CALENDAR_FIRST_WEEKDAY is not None:
    calendar.setfirstweekday(agenda_settings.CALENDAR_FIRST_WEEKDAY)

today = datetime.now()


def event_listing(request, template='agenda/event_list.html', events=None,
    **extra_context):
    """
    View all ``events``.

    If ``events`` is a queryset, clone it. If ``None`` default to all ``Event``s.

    Context parameters:

    events
        an iterable of ``Event`` objects
    """
    if not events:
        events = Event.objects.all()
    elif hasattr(events, '_clone'):
        events = events._clone()

    return render_to_response(template,
        dict(extra_context, events=events),
        context_instance=RequestContext(request))


def event_minimal_view(request, pk, template='agenda/event_minimal_view.html'):
    event = get_object_or_404(Event, pk=pk)
    return render_to_response(template, {'event': event},
        context_instance=RequestContext(request))


def event_category_view(request, slug):
    category = get_object_or_404(EventCategory, slug=slug)
    rdict = {'event_category': category}
    rdict['events'] = Event.objects.filter(event_type__in=[category])
    return render_to_response('agenda/event_category.html', rdict, RequestContext(request))


def agenda_by_category(request, slug):
    agenda = get_object_or_404(Calendar, slug=slug)
    categories = {}
    for cat in EventCategory.objects.all():
        ev = Event.objects.filter(calendar=agenda, event_type=cat)
        if ev.exists():
            categories[cat] = ev
    rdict = {'agenda': agenda, 'events_by_categories': categories}
    print rdict
    return render_to_response('agenda/event_list_by_category.html', rdict, RequestContext(request))


def agenda_by_name(request, slug=None, template='agenda/monthly_view.html', **extra_context):
    return redirect('/agenda/%d/%d/' % (today.year, today.month))


def event_view(request, pk, template='agenda/event_detail.html',
    event_form_class=forms.EventForm,
    recurrence_form_class=forms.MultipleOccurrenceForm):
    """
    View an ``Event`` instance and optionally update either the event or its
    occurrences.

    Context parameters:

    event
        the event keyed by ``pk``

    event_form
        a form object for updating the event

    recurrence_form
        a form object for adding occurrences
    """
    event = get_object_or_404(Event, pk=pk)
    event_form = recurrence_form = None
    if request.method == 'POST':
        if '_update' in request.POST:
            event_form = event_form_class(request.POST, instance=event)
            if event_form.is_valid():
                event_form.save(event)
                return http.HttpResponseRedirect(request.path)
        elif '_add' in request.POST:
            recurrence_form = recurrence_form_class(request.POST)
            if recurrence_form.is_valid():
                recurrence_form.save(event)
                return http.HttpResponseRedirect(request.path)
        else:
            return http.HttpResponseBadRequest('Bad Request')

    event_form = event_form or event_form_class(instance=event)
    if not recurrence_form:
        recurrence_form = recurrence_form_class(
            initial=dict(dtstart=datetime.now())
        )

    return render_to_response(template,
        dict(event=event, event_form=event_form, recurrence_form=recurrence_form),
        context_instance=RequestContext(request))


def occurrence_view(request, event_pk, pk,
    template='agenda/occurrence_detail.html',
    form_class=forms.SingleOccurrenceForm):
    """
    View a specific occurrence and optionally handle any updates.

    Context parameters:

    occurrence
        the occurrence object keyed by ``pk``

    form
        a form object for updating the occurrence
    """
    occurrence = get_object_or_404(Occurrence, pk=pk, event__pk=event_pk)
    if request.method == 'POST':
        form = form_class(request.POST, instance=occurrence)
        if form.is_valid():
            form.save()
            return http.HttpResponseRedirect(request.path)
    else:
        form = form_class(instance=occurrence)

    return render_to_response(template,
        dict(occurrence=occurrence, form=form),
        context_instance=RequestContext(request))


def add_event(request, template='agenda/add_event.html',
    event_form_class=forms.EventForm,
    recurrence_form_class=forms.MultipleOccurrenceForm):
    """
    Add a new ``Event`` instance and 1 or more associated ``Occurrence``s.

    Context parameters:

    dtstart
        a datetime.datetime object representing the GET request value if present,
        otherwise None

    event_form
        a form object for updating the event

    recurrence_form
        a form object for adding occurrences

    """
    dtstart = None
    if request.method == 'POST':
        event_form = event_form_class(request.POST)
        recurrence_form = recurrence_form_class(request.POST)
        if event_form.is_valid() and recurrence_form.is_valid():
            event = event_form.save()
            recurrence_form.save(event)
            return http.HttpResponseRedirect(event.get_absolute_url())

    else:
        if 'dtstart' in request.GET:
            try:
                dtstart = parser.parse(request.GET['dtstart'])
            except:
                # TODO A badly formatted date is passed to add_event
                dtstart = datetime.now()

        event_form = event_form_class()
        recurrence_form = recurrence_form_class(initial=dict(dtstart=dtstart))

    return render_to_response(template,
        dict(dtstart=dtstart, event_form=event_form, recurrence_form=recurrence_form),
        context_instance=RequestContext(request))


def _datetime_view(request, template, dt, timeslot_factory=None,
    items=None, params=None):
    """
    Build a time slot grid representation for the given datetime ``dt``. See
    utils.create_timeslot_table documentation for items and params.

    Context parameters:

    day
        the specified datetime value (dt)

    next_day
        day + 1 day

    prev_day
        day - 1 day

    timeslots
        time slot grid of (time, cells) rows

    """
    timeslot_factory = timeslot_factory or utils.create_timeslot_table
    params = params or {}
    data = dict(
        day=dt,
        next_day=dt + timedelta(days=+1),
        prev_day=dt + timedelta(days=-1),
        timeslots=timeslot_factory(dt, items, **params)
    )

    return render_to_response(template, data,
        context_instance=RequestContext(request))



def day_view(request, year, month, day, template='agenda/daily_view.html', **params):
    """
    See documentation for function``_datetime_view``.
    """
    dt = datetime(int(year), int(month), int(day))
    return _datetime_view(request, template, dt, **params)


def today_view(request, template='agenda/daily_view.html', **params):
    """
    See documentation for function``_datetime_view``.
    """
    return _datetime_view(request, template, datetime.now(), **params)



def year_view(request, year, template='agenda/yearly_view.html', queryset=None):
    """
    Context parameters:

    year
        an integer value for the year in questin

    next_year
        year + 1

    last_year
        year - 1

    by_month
        a sorted list of (month, occurrences) tuples where month is a
        datetime.datetime object for the first day of a month and occurrences
        is a (potentially empty) list of values for that month. Only months
        which have at least 1 occurrence is represented in the list

    """
    year = int(year)
    if queryset:
        queryset = queryset._clone()
    else:
        queryset = Occurrence.objects.select_related()

    occurrences = queryset.filter(
        models.Q(start_time__year=year) | models.Q(end_time__year=year))

    def grouper_key(o):
        if o.start_time.year == year:
            return datetime(year, o.start_time.month, 1)

        return datetime(year, o.end_time.month, 1)

    by_month = [
        (dt, list(items))
        for dt,items in itertools.groupby(occurrences, grouper_key)
    ]

    return render_to_response(template,
        dict(year=year, by_month=by_month, next_year=year + 1, last_year=year - 1),
        context_instance=RequestContext(request))


def month_view(request, year=today.year, month=today.month, template='agenda/monthly_view.html',
    queryset=None):
    """
    Render a traditional calendar grid view with temporal navigation variables.

    Context parameters:

    today
        the current datetime.datetime value

    calendar
        a list of rows containing (day, items) cells, where day is the day of
        the month integer and items is a (potentially empty) list of occurrence
        for the day

    week
        localized list of weekdays, depending on FIRST_DAY_OF_WEEK

    this_month
        a datetime.datetime representing the first day of the month

    next_month
        this_month + 1 month

    last_month
        this_month - 1 month

    """
    year, month = int(year), int(month)
    cal = calendar.monthcalendar(year, month)
    dtstart = datetime(year, month, 1)
    last_day = max(cal[-1])
    dtend = datetime(year, month, last_day)

    # TODO Whether to include those occurrences that started in the previous
    # month but end in this month?
    if queryset:
        queryset = queryset._clone()
    else:
        queryset = Occurrence.objects.select_related()

    occurrences = queryset.filter(start_time__year=year, start_time__month=month)

    by_day = dict([
        (dom, list(items))
        for dom,items in itertools.groupby(occurrences, lambda o: o.start_time.day)
    ])

    weekdays = range(8)
    for wd in forms.WEEKDAY_LONG:
        weekdays[wd[0]] = wd[1]
    weekdays[0] = weekdays[7]

    data = dict(
        today=datetime.now(),
        calendar=[[(d, by_day.get(d, [])) for d in row] for row in cal],
        week=(weekdays[i] for i in range(settings.FIRST_DAY_OF_WEEK, settings.FIRST_DAY_OF_WEEK+7)),
        this_month=dtstart,
        next_month=dtstart + timedelta(days=+last_day),
        last_month=dtstart + timedelta(days=-1),
    )

    return render_to_response(template, data,
        context_instance=RequestContext(request))





# -*- coding:utf-8 -*-

from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization, OrganizationCategory, Person
from coop_geo.models import Location, Area, AreaType
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.conf import settings
from django.http import Http404
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
import json
from coop.models import rdfGraphAll
from django.core.exceptions import ImproperlyConfigured

if('coop.exchange' in settings.INSTALLED_APPS):
    from coop_local.models import Exchange

if('coop.agenda' in settings.INSTALLED_APPS):
    from coop_local.models import Event, EventCategory


def home(request):
    rdict = {}
    rdict['dernieres_initiatives'] = Organization.objects.filter(active=True)[:10]
    if('coop.exchange' in settings.INSTALLED_APPS):
        rdict['dernieres_annonces'] = Exchange.objects.all()[:10]
    return render_to_response('home.html', rdict, RequestContext(request))




# def d2r_mapping(request, mode):
#     if(request.META['REMOTE_ADDR'] in settings.INTERNAL_IPS and mode in ['view', 'export']):
#         db = settings.DATABASES['default']
#         rdict = {}
#         rdict['mode'] = mode
#         rdict['namespaces'] = settings.RDF_NAMESPACES
#         rdict['d2r_baseURI'] = settings.D2RQ_ROOT
#         rdict['d2r_niceURI'] = 'http://' + settings.DEFAULT_URI_DOMAIN + '/'
#         rdict['d2r_port'] = settings.D2RQ_PORT
#         rdict['d2r_site_name'] = unicode(settings.SITE_TITLE) + u' : RDF mapping'
#         rdict['d2rq_username'] = db['USER']
#         rdict['d2rq_password'] = db['PASSWORD']
#         rdict['d2rq_db_name'] = db['NAME']
#         rdict['d2rq_db_port'] = db['PORT'] if db['PORT'] != '' else 5432
#         rdict['d2rq_db_host'] = db['HOST'] if db['HOST'] != '' else 'localhost'
#     else:
#         raise Http404
#     return render_to_response('d2r/mapping.ttl', rdict, RequestContext(request))


from django.db import models
from coop.models import StaticURIModel
from django.db.models.loading import get_model

urimodels = dict((m.__name__.lower(), m.__module__.split('.')[0]) for m in models.get_models() if m.__mro__.__contains__(StaticURIModel))
# To be compliante with the dynamic loading process
for k in urimodels:
    if urimodels[k] == 'coop':
        urimodels[k] = 'coop_local'


RDF_SERIALIZATIONS = {
    'nt': 'text/plain',
    'n3': 'text/n3',
    'ttl': 'text/turtle',
    'xml': 'application/rdf+xml',
    'json': 'application/json',
    'trix': 'application/trix'
}


# smallest data API ever
def get_rdf(request, model, uuid, format):
    req_model = get_model(urimodels[model], model)
    object = get_object_or_404(req_model, uuid=uuid)
    return HttpResponse(object.toRdf(format), mimetype=RDF_SERIALIZATIONS[format])


def rdfdump(request, model, format):
    if model == 'all':
        g = rdfGraphAll()
    else:
        g = rdfGraphAll(model)
    if format == 'ttl':
        return HttpResponse(g.serialize(format='n3'), mimetype=RDF_SERIALIZATIONS[format])
    elif format == 'json':
        return HttpResponse(g.serialize(format='json-ld'), mimetype=RDF_SERIALIZATIONS[format])
    elif format in RDF_SERIALIZATIONS:
        return HttpResponse(g.serialize(format=format), mimetype=RDF_SERIALIZATIONS[format])


def SentryHandler500(request):
    from django.template import Context, loader
    from django.http import HttpResponseServerError

    t = loader.get_template('500.html')  # You need to create a 500.html template.
    return HttpResponseServerError(t.render(Context({
        'request': request,
        'STATIC_URL': settings.STATIC_URL,
    })))


def geojson_objects(request, what, criteria):
    positions = {}
    if what == "org":
        cat = OrganizationCategory.objects.get(slug=criteria)
        qs = Organization.objects.filter(category=cat)
        for org in qs:
            for loc in org.locations():
                if not loc.city in positions:
                    positions[loc.city] = {
                        "type": "Feature",
                        "properties": {
                            #"name": cl.commune.nom,
                            "popupContent": u"<h4>" + loc.city + u"</h4><p class='org'><a href='" + org.get_absolute_url() + u"'>" + org.label() + u"</a></p>"
                            },
                        "geometry": {
                            "type": "Point",
                            #"coordinates": [str(loc.point.x), str(loc.point.y)]
                            "coordinates": [loc.point.x, loc.point.y]

                            }
                        }

                else:
                    positions[loc.city]["properties"]["popupContent"] += u"\n<p class='geo_item'><a href='" + \
                            org.get_absolute_url() + u"'>" + org.label() + u"</a></p>"
    elif what == "event":
        cat = EventCategory.objects.get(slug=criteria)
        qs = Event.objects.filter(event_type=cat)  # TODO rename category !!

        for event in qs:
            loc = event.location
            if loc and not loc.city in positions:
                positions[loc.city] = {
                    "type": "Feature",
                    "properties": {
                        "popupContent": u"<h4>" + loc.city + u"</h4><p class='org'><a href='" + event.get_absolute_url() + u"'>" + event.title + u"</a></p>"
                        },
                    "geometry": {
                        "type": "Point",
                        "coordinates": [loc.point.x, loc.point.y]

                        }
                    }
            else:
                positions[loc.city]["properties"]["popupContent"] += u"\n<p class='geo_item'><a href='" + \
                            event.get_absolute_url() + u"'>" + event.title + u"</a></p>"

    result = {"type": "FeatureCollection", "features": positions.values()}
    return HttpResponse(json.dumps(result), mimetype="application/json")


from django.core import serializers


def communes(request):
    label = request.GET['term']
    qs = Location.objects.filter(city__icontains=label, is_ref_center=True)
    data = []
    for l in qs:
        item = {}
        item['label'] = l.city
        item['value'] = l.point.coords
        data.append(item)

    #data = serializers.serialize('json', qs, fields=('city','point'))

    return HttpResponse(json.dumps(data), mimetype="application/json")


def geojson(request):

    from django.contrib.gis.measure import Distance
    from django.contrib.gis.geoip import GeoIP
    from django.contrib.gis.geos import Point
    from django.contrib.gis.geos import fromstr

    #import pdb; pdb.set_trace()

    if request.GET.get('distance'):
        dist = request.GET['distance']
    else:
        dist = 20

    if request.GET.get('center'):
        print request.GET['center']
        coords = request.GET['center'].split(',')
        my_lat = coords[0]
        my_long = coords[1]
        center = fromstr('POINT(' + my_lat + " " + my_long + ')')
    else:
        g = GeoIP()
        center = g.geos(request.META['REMOTE_ADDR'])

    try:
        regiontype = AreaType.objects.get(label="Région")
        region = Area.objects.get(area_type=regiontype, reference=settings.DEFAULT_REGION_CODE)
    except:
        raise ImproperlyConfigured("No default region has been configured")

    if not center or not region.polygon.contains(center):
        center = region.default_location.point
        dist = 1000

    res = []

    for location in Location.objects.filter(point__distance_lte=(center, Distance(km=dist))):
        for loc in location.located_set.all():
            obj = loc.content_object
            if obj.__class__ == Organization and "amap" in [x.slug for x in obj.category.all()]:
                if obj.to_geoJson():
                    res.append(obj.to_geoJson())

    result = {"type": "FeatureCollection", "features":  res}
    return HttpResponse(json.dumps(result), mimetype="application/json")




# -*- coding:utf-8 -*-

from django import template
from urlparse import urlparse, urlunparse
from django.core.exceptions import FieldError
from django.db.models.fields import FieldDoesNotExist

register = template.Library()

@register.simple_tag
def favicon(url):
    parsed_url = urlparse(url)
    return urlunparse((parsed_url[0], parsed_url[1],
                       u'favicon.ico', parsed_url[3],
                       parsed_url[4], parsed_url[5]))


class LocalRemoteNode(template.Node):

    def __init__(self, attr, obj, model_name, as_var=None):
        self.attr = attr
        self.obj = template.Variable(obj)
        self.model_name = model_name
        if as_var:
            self.as_var = as_var

    def render(self, context):
        try:
            object = self.obj.resolve(context)
        except template.VariableDoesNotExist:
            return ''
        try:
            object._meta.get_field_by_name(self.model_name)
        except(FieldDoesNotExist):
            raise template.TemplateSyntaxError('Unknown attribute for %s' % object)

        related = object.__getattribute__(self.model_name)
        if related:
            # l'objet a bien une Fkey vers le modele correspondant
            # on renvoie donc le champ lié demandé
            if self.attr == 'uri':
                res = related.get_absolute_url()
            elif self.attr == 'label':
                res = related.label()
        else:
            # pas de Fkey donc on cherche un champ remote_{nom du modèle lié}_uri
            try:
                fieldname = 'remote_' + self.model_name + '_' + self.attr
                res = object.__getattribute__(fieldname)
            except AttributeError:
                raise FieldError('Field remote_%s_%s is not present on %s, correct your templatetag call syntax or add the corresponding field' % (self.model_name, self.attr, object))
        if self.as_var:
            context[self.as_var] = res
            return u''
        else:
            return res


@register.tag
def local_or_remote(parser, token):

    contents = token.split_contents()
    params = contents[1:]
    try:
        if len(params) == 3:
            tag_name, obj, model_name = params
        elif len(params) == 5 and params[3] == u'as':
            tag_name, obj, model_name, as_param, as_var = params
            params.remove(u'as')
    except ValueError:
        raise template.TemplateSyntaxError, "This tag requires three arguments : object, uri or label, linked model name, and optional as parameter with a variable name"
    #TODO verifier ce qu'on a
    return LocalRemoteNode(*params)




# -*- coding:utf-8 -*-

from django.conf import settings
from django.contrib import admin
from django import forms
from coop_local.models import Event, EventCategory, Calendar, Occurrence, Dated
from coop.utils.autocomplete_admin import FkAutocompleteAdmin, InlineAutocompleteAdmin, NoLookupsFkAutocompleteAdmin
from coop_geo.admin import LocatedInline
from django.db.models.loading import get_model
from django.contrib.contenttypes.generic import GenericTabularInline
from django.utils.translation import ugettext_lazy as _
from coop.agenda.forms import SingleOccurrenceForm

#from genericadmin.admin import GenericAdminModelAdmin
# GenericStackedInline or GenericTabularInline
# on fera le lien generique depuis l'objet en question si besoin (article, événement)


class EventCategoryAdmin(admin.ModelAdmin):
    list_display = ('label',)


class OccurrenceInline(admin.StackedInline):
    #form = SingleOccurrenceForm
    verbose_name = _(u'Date')
    verbose_name_plural = _(u'Dates')
    model = Occurrence
    extra = 1


class EventAdminForm(forms.ModelForm):
    class Meta:
        model = get_model('coop_local', 'Event')
        #widgets = {'category': chosenwidgets.ChosenSelectMultiple()}

    def __init__(self, *args, **kwargs):
        # initial = {
        #     'event_type': 1, 
        #     'calendar': 1 
        #     }
        # if not kwargs.has_key('initial'):
        # #     kwargs['initial'].update(initial)
        # # else:
        #     kwargs['initial'] = initial
        # Initializing form only after you have set initial dict
        super(EventAdminForm, self).__init__(*args, **kwargs)
        self.fields['calendar'].initial = Calendar.objects.get(id=1)
        self.fields['event_type'].initial = EventCategory.objects.get(id=1)


class EventAdmin(NoLookupsFkAutocompleteAdmin):
    change_form_template = 'admintools_bootstrap/tabbed_change_form.html'
    form = EventAdminForm
    list_display = ('title', 'event_type', 'description')
    list_filter = ('event_type', )
    search_fields = ('title', 'description')
    related_search_fields = {'person': ('last_name', 'first_name',
                                        'email', 'structure', 'username'),
                            'organization': ('title', 'acronym', 'subtitle', 'description'),
                            'location': ('label', 'adr1', 'adr2', 'zipcode', 'city'),
    }
    fieldsets = [['Description', {'fields': ['title', 'description',
        ('event_type', 'calendar'),
        'organization', 'remote_organization_label', 'remote_organization_uri',
        'person', 'remote_person_label', 'remote_person_uri',
        'location', 'remote_location_label', 'remote_location_uri',
      ]}],
    ]

    if "coop_tag" in settings.INSTALLED_APPS:
        fieldsets[0][1]['fields'].insert(2, 'tags')

    inlines = [OccurrenceInline]

admin.site.register(Event, EventAdmin)
admin.site.register(EventCategory, EventCategoryAdmin)
admin.site.register(Calendar)


class DatedInline(GenericTabularInline, InlineAutocompleteAdmin):
    verbose_name = _(u'Date')
    verbose_name_plural = _(u'Dates')
    model = Dated
    related_search_fields = {'event': ('title', 'description', 'slug'), }
    extra = 1



# -*- coding:utf-8 -*-
from django import forms
import floppyforms
import re
from coop_local.models import Organization, OrganizationCategory
from djaloha.widgets import AlohaInput
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError


class OrganizationForm(floppyforms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(OrganizationForm, self).__init__(*args, **kwargs)
        self.org = kwargs.get('instance', None)
        #self.set_logo_size()

    class Meta:
        model = Organization
        fields = ('title', 'description', 'logo')
        widgets = {
            'title': AlohaInput(text_color_plugin=False),
            'description': AlohaInput(text_color_plugin=False),
        }

    # def set_logo_size(self, logo_size=None):
    #     thumbnail_src = self.logo_thumbnail(logo_size)
    #     update_url = reverse('coop_cms_update_logo', args=[self.article.id])
    #     self.fields['logo'].widget = ImageEdit(update_url, thumbnail_src.url if thumbnail_src else '')

    def logo_thumbnail(self, logo_size=None):
        if self.org:
            return self.org.logo_thumbnail(True, logo_size=logo_size)

    def clean_title(self):
        title = self.cleaned_data['title'].strip()
        if title[-4:].lower() == '<br>':
            title = title[:-4]
        if not title:
            raise ValidationError(_(u"Title can not be empty"))
        if re.search(u'<(.*)>', title):
            raise ValidationError(_(u'HTML content is not allowed in the title'))
        return title


class OrganizationCategoryForm(floppyforms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(OrganizationCategoryForm, self).__init__(*args, **kwargs)
        self.org_category = kwargs.get('instance', None)

    class Meta:
        model = OrganizationCategory
        fields = ('label', 'description')
        widgets = {
            'label': AlohaInput(text_color_plugin=False),
            'description': AlohaInput(text_color_plugin=False),
        }

    def clean_label(self):
        label = self.cleaned_data['label'].strip()
        if label[-4:].lower() == '<br>':
            label = label[:-4]
        if not label:
            raise ValidationError(_(u"Label can not be empty"))
        if re.search(u'<(.*)>', label):
            raise ValidationError(_(u'HTML content is not allowed in the label'))
        return label





# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.template.loader import get_template
from django.template import Context
from django.conf import settings
from coop_bar.utils import make_link
from django.contrib.contenttypes.models import ContentType


def coop_perm(perm, object_names):
    def inner_decorator(func):
        def wrapper(request, context):
            editable = context.get('editable')
            # objet editable en front-end ?
            if not editable:
                return
            for model_name in object_names:
                # objet  present dans le contexte ?
                object = context.get(model_name)
                if object and request and request.user.has_perm(perm + "_" + model_name, object):
                    yes_we_can = func(request, context)
                    if yes_we_can:
                        return yes_we_can
            return
        return wrapper
    return inner_decorator

# @can_post_event
# def post_event(request, context):
#     org = context.get('organization')
#     url = reverse('org_detail', args=[org.slug])
#     return make_link(url, _(u'Post new event'), 'fugue/calendar--plus.png.png',classes=['icon'])


from coop.org.coop_bar_links import *
from coop_cms.coop_bar_cfg import *


def admin_articles(request, context):
    if request and request.user.is_staff:
        return make_link(reverse('admin:coop_local_article_changelist'), 'Articles', 'fugue/documents-stack.png',
            classes=['icon', 'alert_on_click'])


def load_commands(coop_bar):
    coop_bar.register([

        [django_admin, admin_articles],  # django_admin_navtree

        [   org_edit_link,
            org_view_link,
            org_save,
            org_edit_cancel_link,
            org_admin_link
        ],
        [   org_category_edit_link,
            org_category_view_link,
            org_save,
            org_category_edit_cancel_link
        ],

        [cms_edit, cms_view, cms_save, cms_cancel, django_admin_edit_article, cms_article_settings],

        [cms_new_article, ],

        [cms_new_newsletter, edit_newsletter, cancel_edit_newsletter, save_newsletter,
            change_newsletter_settings,
            schedule_newsletter, test_newsletter],

        [cms_media_library, cms_upload_image, cms_upload_doc],

        [log_out]
    ])




# -*- coding:utf-8 -*-

from django.contrib import admin
from coop_tag.settings import get_class
#from feincms.admin import tree_editor

# TODO : find a nice generic object widget display
# class TaggedItemInline(admin.StackedInline):
#     model = get_class('taggeditem')


# We just override the TreeEditor template to move columns
# CSS added in coop_tag is here in our admin css sheet

# class CoopTagTreeAdmin(tree_editor.TreeEditor):

#     def changelist_view(self, request, extra_context=None, *args, **kwargs):
#         if 'actions_column' not in self.list_display:
#             self.list_display.insert(0, 'actions_column')
#         return super(CoopTagTreeAdmin, self).changelist_view(request, extra_context)

    # inlines = [
    #     TaggedItemInline
    # ]




# -*- coding:utf-8 -*-
from optparse import make_option
from django.core.management.base import BaseCommand
import coop


class Command(BaseCommand):
    help = "Dump in rdf format all data."
    base_options = (
        make_option('--format', dest='rdfformat', default="xml",
            help='The rdf format (default is xml), possible values are "n3", "json", and "ttl".'
        ),
        make_option("-o", "--output", dest="output_file", default="all",
            help='The output file.'
        ),
        make_option("-m", "--model", dest="model", default=None,
            help='A specif model, default value is None and in this case objects from all models are dump.'
        ),
    )
    option_list = BaseCommand.option_list + base_options

    def handle(self, **options):
        format = options.get('rdfformat')
        dest = options.get('output_file')
        model = options.get('model')
        if not model == None:
            model = model.lower()
        coop.models.rdfDumpAll(dest, format, model)




# -*- coding:utf-8 -*-
from haystack import indexes
from coop.search_indexes import OrganizationIndex as BaseOrganizationIndex
from coop.search_indexes import ExchangeIndex as BaseExchangeIndex
from coop.search_indexes import ArticleIndex as BaseArticleIndex
from coop.search_indexes import EventIndex as BaseEventIndex



# If you dont want to use the default index, please comment the following
# lines and write your own indexes



class OrganizationIndex(BaseOrganizationIndex, indexes.Indexable):
    pass


class ExchangeIndex(BaseExchangeIndex, indexes.Indexable):
    pass


class ArticleIndex(BaseArticleIndex, indexes.Indexable):
    pass


class EventIndex(BaseEventIndex, indexes.Indexable):
    pass



# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, url
from django.contrib.auth import views as auth_views

urlpatterns = patterns('',

    (r'^login/$', 'django.contrib.auth.views.login', {'template_name': 'accounts/login.html'}),
    (r'^logout/$', 'django.contrib.auth.views.logout', {'template_name': 'accounts/logout.html'}),

    # interception de la page edition de profil pour injecter notre formulaire
    # (r'^edit/$', 'profiles.views.edit_profile', {'form_class': ProfileForm,}),

    url(r'^(?P<uuid>\w+)/$', 'coop.person.views.public_profile'),

    url(r'^password/change/$', auth_views.password_change, name='auth_password_change'),
    url(r'^password/change/done/$', auth_views.password_change_done, name='auth_password_change_done'),
    url(r'^password/reset/$', auth_views.password_reset, name='auth_password_reset'),
    url(r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', auth_views.password_reset_confirm, name='auth_password_reset_confirm'),
    url(r'^password/reset/complete/$', auth_views.password_reset_complete, name='auth_password_reset_complete'),
    url(r'^password/reset/done/$', auth_views.password_reset_done, name='auth_password_reset_done'),

    # sinon on délégue à l'app profiles
    # (r'^', include('profiles.urls')),

)



# -*- coding:utf-8 -*-

from django.test import TestCase
from django.utils.translation import ugettext_lazy as _
from django.test.client import Client
from django.db.models import get_app, get_models
from django.conf import settings
from django.contrib import admin

class AdminTest(TestCase):

    fixtures = ['tests/users.json']


    def test_site(self):
        response = self.client.get('/admin/', follow=True)
        self.assertEqual(response.status_code,200, 'check /admin/ without user')

        response = self.client.get('/')
        self.assertEqual(response.status_code,200)


    def test_admin(self):
        self.client.login(username='clo',password='clo')
        response = self.client.get('/admin/')
        self.assertEqual(response.status_code,200)

        app_name = 'coop_local'
        app = get_app(app_name)
        registered = admin.site._registry
        for m in get_models(app):
            if m in registered:
                url = '/admin/%s/%s/' % (app_name,  m.__name__.lower())
                # print 'test url %s' % url
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code,200, "check url %s" % url)










# -*- coding:utf-8 -*-
"""
Hourly cleanup job.

Can be run as a cronjob to clean out old idle RQ workers. The point with RQ workers
is that they need two SIGINT signals to die this is not possible with supervisor
or other circus.... some workers may stay idle It is a good idea to clean them;
"""

import logging
from django.conf import settings
from redis import Redis
from django_extensions.management.jobs import HourlyJob
from rq import Queue, use_connection, get_current_connection, Worker

# As subhub is the only app the use redis. All Redis stuff are logger 
# subhub.mainteance  log
log = logging.getLogger('subhub.maintenance')


class Job(HourlyJob):
    help = "RQ idle workers cleanup Job"

    def execute(self):
        # It is always possible that the Redis connection is not yet set 
        print "ENTER"
        if not get_current_connection():
            conn = Redis('localhost', settings.REDIS_PORT)
            use_connection(conn)
        if not get_current_connection():
            log.error(u'Unable to create redis connection')
        # use the 'default' queue. We only used this one;
        q = Queue()
        # if the queue is not empty then some old idle workers may have to be cleaned
        if not q.is_empty():
            for w in Worker.all():
                if w.state == 'idle' and q in w.queues:
                    log.info(u'Work %s will die gently' % w.name)
                    w.register_death()



# -*- coding:utf-8 -*-

from django.conf.urls.defaults import patterns, include, url
from django.views.generic.base import TemplateView, RedirectView
from django.conf import settings
from coop.feeds import UpdateFeed


class TextPlainView(TemplateView):
    def render_to_response(self, context, **kwargs):
        return super(TextPlainView, self).render_to_response(
              context, content_type='text/plain', **kwargs)

urlpatterns = patterns('',

    url(r'^$', 'coop.views.home', name="home"),

    url(r'^org/', include('coop.org.urls')),
    url(r'^geojson/(?P<what>[\w-]+)/(?P<criteria>[\w-]+)/$', 'coop.views.geojson_objects'),

    url(r'^comments/', include('django.contrib.comments.urls')),

    # a revoir
    (r'^profil/', include('coop.person.urls')),

    url(r'^feed/(?P<model>[\w-]+)/$', UpdateFeed()),
    url(r'^hub/', include('subhub.urls')),
    url(r'^subscriber/', include('django_push.subscriber.urls')),  # Callback

    url(r'^robots\.txt$', TextPlainView.as_view(template_name='robots.txt')),
    url(r'^favicon\.ico$', RedirectView.as_view(url='/static/img/favicon.ico')),
    url(r'^rdfdump/(?P<model>[\w-]+).(?P<format>[\w-]+)$', 'coop.views.rdfdump'),

    # url(r'^d2r/(?P<mode>[\w-]+)/mapping.ttl$', 'coop.views.d2r_mapping'),

    url(r'^communes/$', 'coop.views.communes'),
    url(r'^geojson/', 'coop.views.geojson'),

)

if 'coop.exchange' in settings.INSTALLED_APPS:
    urlpatterns = urlpatterns + patterns('',
        (r'^annonces/', include('coop.exchange.urls')),
)

if 'coop.webid' in settings.INSTALLED_APPS:
    urlpatterns = urlpatterns + patterns('',
        (r'^webid/', include('coop.webid.urls')),
)



# -*- coding:utf-8 -*-
from django.conf import settings
from haystack import indexes
from coop_local.models import Organization, Exchange, Article, Event



if getattr(settings, 'HAYSTACK_REALTIME', False):
    Indexes = indexes.RealTimeSearchIndex
else:
    Indexes = indexes.SearchIndex


# The main class
class CoopIndex(Indexes):
    text = indexes.CharField(document=True, use_template=True)
    tags = indexes.MultiValueField(boost=1.2, faceted=True)
    # modified = indexes.DateField(model_attr='modified', faceted=True)
    rendered = indexes.CharField(use_template=True, indexed=False)

    def prepare_tags(self, obj):
        return [u"%s" % tag.name for tag in obj.tags.all()]

    def prepare(self, obj):
        print "prepare id=%s %s" % (obj.id, obj)
        if obj.title == u'Tous candidats le 2 mars 2012':
            print 'before super'
        prepared_data = super(CoopIndex, self).prepare(obj)
        if obj.title == u'Tous candidats le 2 mars 2012':
            print 'after super'
       
        prepared_data['text'] = prepared_data['text'] + ' ' + \
        ' '.join(prepared_data['tags']) 
        return prepared_data


class OrganizationIndex(CoopIndex):

    def get_model(self):
        return Organization


class ExchangeIndex(CoopIndex):
    product = indexes.MultiValueField(boost=1.2, faceted=True)

    def prepare_product(self, obj):
        return [u"%s" % prod.title for prod in obj.products.all()]

    def prepare(self, obj):
        # print "prepare %s" % obj
        prepared_data = super(CoopIndex, self).prepare(obj)
        prepared_data['text'] = prepared_data['text'] + ' ' + \
        ' '.join(prepared_data['product']) 
        return prepared_data
 
    def get_model(self):
        return Exchange


class ArticleIndex(CoopIndex):

    def get_model(self):
        return Article


class EventIndex(CoopIndex):

    def get_model(self):
        return Event










# -*- coding:utf-8 -*-
from django.db import models
from django.db.models import Count
from autoslug import AutoSlugField
from extended_choices import Choices
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.conf import settings
from coop.models import URIModel
from sorl.thumbnail import ImageField
from sorl.thumbnail import default
import rdflib
import coop
from django.contrib.sites.models import Site
import logging
from urlparse import urlsplit

ADMIN_THUMBS_SIZE = '60x60'

#from mptt.models import MPTTModel, TreeForeignKey
# class BaseClassification(MPTTModel, URIModel):
#     label = models.CharField(_(u'label'), max_length=60)
#     slug = AutoSlugField(populate_from='label', always_update=True, unique=True)
#     parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
#     domain_name = 'data.economie-solidaire.fr'

#     class MPTTMeta:
#         order_insertion_by = ['label']

#     class Meta:
#         abstract = True
#         verbose_name = _('Classification')
#         verbose_name_plural = _('Classifications')
#         ordering = ['tree_id', 'lft']  # for FeinCMS TreeEditor
#         app_label = 'coop_local'

#     def __unicode__(self):
#         return unicode(self.label)

#     def get_absolute_url(self):
#         return reverse('%s-detail' % self._meta.object_name.lower(), args=[self.slug])

#     @property
#     def uri_id(self):
#         return self.slug

#     def uri_registry(self):
#         return u'label'


class BaseRoleCategory(models.Model):
    label = models.CharField(_(u'label'), max_length=60)
    slug = exfields.AutoSlugField(populate_from=('label'), overwrite=True)
    uri = models.CharField(_(u'URI'), blank=True, max_length=250)

    def save(self, *args, **kwargs):
        self.save_base()
        self.uri = u'http://data.economie-solidaire.fr/id/role/%s/' % self.slug
        super(BaseRoleCategory, self).save(*args, **kwargs)

    class Meta:
        abstract = True
        ordering = ['label']
        verbose_name = _('Role category')
        verbose_name_plural = _('Role categories')
        #ordering = ['label']
        app_label = 'coop_local'

    def __unicode__(self):
        return unicode(self.label)


class BaseRole(URIModel):
    label = models.CharField(_(u'label'), max_length=120)
    slug = AutoSlugField(populate_from='label', always_update=True, unique=True, editable=False)
    category = models.ForeignKey('coop_local.RoleCategory', null=True, blank=True, verbose_name=_(u'category'))

    domain_name = 'data.economie-solidaire.fr'

    class Meta:
        abstract = True
        verbose_name = _('Role')
        verbose_name_plural = _('Roles')
        #ordering = ['tree_id', 'lft']  # for FeinCMS TreeEditor
        #ordering = ['label']
        app_label = 'coop_local'

    @property
    def uri_id(self):
        return self.slug

    def uri_registry(self):
        return u'label'

    def __unicode__(self):
        return unicode(self.label)

    def get_absolute_url(self):
        return reverse('role_detail', args=[self.slug])

    # rdf stuff
    rdf_type = settings.NS.org.Role
    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.skos.prefLabel, 'label'), 'single_reverse'),

        # quand RoleCategory sera un URIModels on aura tout simpleùent
        #('single_mapping', (settings.NS.skos.broader, 'category'), 'single_reverse'),

        ('category_mapping', (settings.NS.skos.broader, 'category'), 'category_mapping_reverse'),
    ]

    def category_mapping(self, rdfPred, djF, lang=None):
        value = getattr(self, djF)
        if value == None:
            return []
        else:
            return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.URIRef(value.uri))]

    def category_mapping_reverse(self, g, rdfPred, djF, lang=None):
        values = list(g.objects(rdflib.term.URIRef(self), rdfPred))
        if values == []:
            setattr(self, djF, None)
        elif len(values) == 1:
            value = values[0]
            setattr(self, djF, models.get_model('coop_base', 'rolecategory').object.get(uri=value))


# will apply to contact numbers and other things
# TODO simplify it, see PPO Ontology

DISPLAY = Choices(
    ('PUBLIC',  1,  _(u'public information')),
    ('USERS',   2,  _(u'registered users')),
    ('MEMBER',  3,  _(u'members of an organization')),
    ('RELATED', 4,  _(u'related organizations members')),
    ('ORG',     5,  _(u'members of this organization')),
    ('ADMIN',   6,  _(u'administrators of this site')),
)

COMM_MEANS = Choices(
    ('MAIL',    8,  _(u'E-mail')),
    ('LAND',    1,  _(u'Landline phone')),
    ('GSM',     2,  _(u'Mobile phone')),
    ('FAX',     3,  _(u'Fax')),
    ('WEB',     9,  _(u'Secondary web site')),
    ('SKYPE',   4,  _(u'Skype')),
    ('TWITTER', 5,  _(u'Twitter')),
    ('RSS',     6,  _(u'RSS Feed')),
    ('VCAL',    7,  _(u'vCalendar')),
)


class BaseContactMedium(models.Model):  # this model will be initialized with a fixture
    label = models.CharField(_(u'label'), max_length=250)
    uri = models.CharField(_(u'URI'), blank=True, max_length=250)

    def __unicode__(self):
        return self.label

    class Meta:
        abstract = True
        verbose_name = _(u'Contact medium')
        verbose_name_plural = _(u'Contact mediums')
        app_label = 'coop_local'


class BaseContact(URIModel):
    """ A model which represents any communication medium (a phone number, an email) """
    category = models.PositiveSmallIntegerField(_(u'Category'),
                    choices=COMM_MEANS.CHOICES, editable=False, blank=True, null=True)  # TODO erase when data migrated
    contact_medium = models.ForeignKey('coop_local.ContactMedium', blank=True, null=True, verbose_name=u'medium')

    content = models.CharField(_(u'content'), max_length=250)
    details = models.CharField(_(u'details'), blank=True, max_length=100)
    display = models.PositiveSmallIntegerField(_(u'Display'),
                    choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)

    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True
        ordering = ['category']
        verbose_name = _(u'Contact')
        verbose_name_plural = _(u'Contacts')
        app_label = 'coop_local'

    def __unicode__(self):
        if self.content_object != None:
            return self.content + u' (' + self.content_object.__unicode__() + u')'
        else:
            return self.content

    def label(self):
        return self.__unicode__()

    @classmethod
    def clean_phone(cls, number):
        import re
        phoneSplitRegex = re.compile(r"[\-\(\) \.\,]")
        parts = phoneSplitRegex.split(number)
        num = ''.join(parts)
        if len(num) == 10:
            cleaned_number = '.'.join((num[:2], num[2:4], num[4:6], num[6:8], num[8:]))
            return cleaned_number
        else:
            raise ValidationError(_(u'Phone numbers must have 10 digits'))

    def save(self, *args, **kwargs):
        if self.category in [1, 2, 3]:
            self.content = self.clean_phone(self.content)
        #logging.error(u'A contact has been created or modified', exc_info=True, extra={'request': request})
        super(BaseContact, self).save(*args, **kwargs)



    # RDF stuff

    rdf_type = settings.NS.ess.ContactMedium

    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.rdf.value, 'content'), 'single_reverse'),
        ('single_mapping', (settings.NS.rdfs.comment, 'details'), 'single_reverse'),

        ('medium_mapping', (settings.NS.rdf.type, 'contact_medium'), 'medium_mapping_reverse'),
    ]


    def medium_mapping(self, rdfPred, djF, lang=None):
        medium = getattr(self, djF)
        rdfSubject = rdflib.URIRef(self.uri)
        return [(rdfSubject, rdfPred, rdflib.URIRef(medium.uri))]


    def medium_mapping_reverse(self, g, rdfPred, djF, lang=None):
        values = list(g.objects((rdflib.term.URIRef(self.uri), rdfPred)))        
        values.remove(self.rdf_type)
        m = models.get_model('coop_local', 'contactmedium')
        if len(values) == 1:
            value = values[0]
            medium = m.objects.get(uri=str(value))
            setattr(self, djF, medium)


# TODO : use django-multilingual-ng to translate the label field in multiple languages

class BaseOrgRelationType(models.Model):  # this model will be initialized with a fixture
    label = models.CharField(_(u'label'), max_length=250)
    uri = models.CharField(_(u'URI'), blank=True, max_length=250)

    def __unicode__(self):
        return self.label

    class Meta:
        abstract = True
        verbose_name = _(u'Organizations relation type')
        verbose_name_plural = _(u'Organization relations types')
        app_label = 'coop_local'


RELATIONS = Choices(
    ('MEMBER',          1,  _(u' is member of ')),
    ('REG_SUPPLIER',    2,  _(u' has for regular supplier ')),
    ('OCC_SUPPLIER',    3,  _(u' has for casual supplier ')),
    ('SUPPORT',         4,  _(u' received technical support from ')),
    ('FUNDING',         5,  _(u' received financial support from ')),
)


class BaseRelation(models.Model):
    source = models.ForeignKey('coop_local.Organization', verbose_name=_(u'source organization'), related_name='source')
    target = models.ForeignKey('coop_local.Organization', verbose_name=_(u'target organization'), related_name='target')
    reltype = models.PositiveSmallIntegerField(_(u'Relation type'), choices=RELATIONS.CHOICES, blank=True, null=True)  # TODO erase when data migrated AND remove
    relation_type = models.ForeignKey('coop_local.OrgRelationType', verbose_name=_(u'relation type'), 
                                                            null=True, blank=True)      # blank, null = True from the new Fkey
    
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    #confirmed = models.BooleanField(default=False, verbose_name=_(u'confirmed by the target organization'))

    class Meta:
        abstract = True
        verbose_name = _('Relation')
        verbose_name_plural = _('Relations')
        app_label = 'coop_local'

    def __unicode__(self):
        return u'"' + self.source.__unicode__() + u'"' + \
            self.relation_type.__unicode__() + \
            u'"' + self.target.__unicode__() + u'".'
    '''
    def save(self):
        vérifier si la relation inverse existe
    '''


class BaseEngagement(URIModel):
    person = models.ForeignKey('coop_local.Person', verbose_name=_(u'person'), related_name='engagements')
    organization = models.ForeignKey('coop_local.Organization', verbose_name=_(u'organization'))
    role = models.ForeignKey('coop_local.Role', verbose_name=_(u'role'), null=True, blank=True)
    role_detail = models.CharField(_(u'detailed role'), blank=True, max_length=100)
    org_admin = models.BooleanField(_(u'has editor rights'), default=True)
    engagement_display = models.PositiveSmallIntegerField(_(u'Display'), choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)

    remote_role_uri = models.URLField(_(u'URI'), blank=True, null=True, max_length=250)
    remote_role_label = models.CharField(blank=True, max_length=100)

    remote_organization_uri = models.URLField(_(u'remote organization URI'), blank=True, max_length=255, editable=False)
    remote_organization_label = models.CharField(_(u'remote organization label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the organization record is not available locally'))


    class Meta:
        abstract = True
        verbose_name = _('Engagement')
        verbose_name_plural = _('Engagements')
        app_label = 'coop_local'

    def __unicode__(self):
        return '%(person)s, %(role)s @ %(org)s' % {
                    'person': self.person.__unicode__(),
                    'role': self.role.__unicode__(),
                    'org': self.organization.__unicode__()
                    }

    def label(self):
        return self.__unicode__()

    # RDF stufs
    rdf_type = settings.NS.org.Membership
    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.org.member, 'person'), 'single_reverse'),

        ('local_or_remote_mapping', (settings.NS.org.organization, 'organization'), 'local_or_remote_mapping'),
        ('local_or_remote_mapping', (settings.NS.org.role, 'role'), 'local_or_remote_mapping'),

        ('label_mapping', (settings.NS.rdfs.label, 'id', 'fr'), 'label_mapping_reverse'),

    ]

    def label_mapping(self, rdfPred, djF, lang):
        return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(u'Engagement n°%s' % self.id, lang))]

    def label_mapping_reverse(self, g, rdfPred, djF, lang=None):
        pass



class BaseOrganizationCategory(models.Model):
    label = models.CharField(blank=True, max_length=100)
    slug = exfields.AutoSlugField(populate_from=('label'), overwrite=True)
    description = models.TextField(_(u'description'), blank=True)

    class Meta:
        abstract = True
        verbose_name = _(u'organization category')
        verbose_name_plural = _(u'organization categories')
        app_label = 'coop_local'

    def __unicode__(self):
        return self.label

    #@models.permalink
    def get_absolute_url(self):
        return reverse('org_category_detail', args=[self.slug])

    def get_edit_url(self):
        return reverse('org_category_edit', args=[self.slug])

    def get_cancel_url(self):
        return reverse('org_category_edit_cancel', args=[self.slug])

    def _can_modify_organizationcategory(self, user):
        if user.is_authenticated():
            if user.is_superuser:
                return True
            else:
                return False

    def can_view_organizationcategory(self, user):
        # TODO use global privacy permissions on objects
        return True

    def can_edit_organizationcategory(self, user):
        return self._can_modify_organizationcategory(user)







PREFLABEL = Choices(
    ('TITLE',   1,  _(u'title')),
    ('ACRO',    2,  _(u'acronym')),
)


def get_logo_folder(self, filename):
    img_root = 'org_logos'
    return u'{0}/{1}/{2}'.format(img_root, self.id, filename)


class BaseOrganization(URIModel):
    title = models.CharField(_(u'title'), max_length=250)

    acronym = models.CharField(_(u'acronym'), max_length=100, blank=True, null=True)
    pref_label = models.PositiveSmallIntegerField(_(u'Preferred label'),
                        choices=PREFLABEL.CHOICES, default=PREFLABEL.TITLE)

    subtitle = models.CharField(_(u'tagline'), blank=True, null=True,
                max_length=250,
                help_text=_(u'tell us what your organization do in one line.'))

    description = models.TextField(_(u'description'), blank=True, null=True)

    logo = ImageField(upload_to='logos/', null=True, blank=True)
    #temp_logo = models.ImageField(upload_to=get_logo_folder, blank=True, null=True, default='')


    relations = models.ManyToManyField('coop_local.Organization',
                symmetrical=False, through='coop_local.Relation',
                verbose_name=_(u'relations'))

    category = models.ManyToManyField('coop_local.OrganizationCategory',
                blank=True, null=True, verbose_name=_(u'category'))

    members = models.ManyToManyField('coop_local.Person',
                through='coop_local.Engagement', verbose_name=_(u'members'))

    contacts = generic.GenericRelation('coop_local.Contact')

    if 'coop.mailing' in settings.INSTALLED_APPS:
        subs = generic.GenericRelation('coop_local.Subscription')

    # ORDER : coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:
        located = generic.GenericRelation('coop_geo.Located')  # , related_name='located_org')
        framed = generic.GenericRelation('coop_geo.AreaLink')  # , related_name='framed_org')

    birth = models.DateField(_(u'creation date'), null=True, blank=True)
    email = models.EmailField(_(u'global email'), blank=True, null=True)
    email_sha1 = models.CharField(_(u'email checksum'),
            max_length=250, blank=True, null=True)  # TODO : do this in Postgre
    web = models.URLField(_(u'web site'), blank=True, null=True)

    pref_email = models.ForeignKey('coop_local.Contact',
                verbose_name=_(u'preferred email'),
                related_name="pref_email", null=True, blank=True)
    pref_phone = models.ForeignKey('coop_local.Contact',
                verbose_name=_(u'preferred phone'),
                related_name='pref_phone', null=True, blank=True)
    pref_address = models.ForeignKey('coop_geo.Location',
                verbose_name=_(u'preferred postal address'),
                related_name='pref_adress', null=True, blank=True)

    slug = exfields.AutoSlugField(populate_from='title', blank=True, overwrite=True)
    active = models.BooleanField(_(u'show on public site'), default=True,)
    notes = models.TextField(_(u'notes'), blank=True)

    if "coop.agenda" in settings.INSTALLED_APPS:
        dated = generic.GenericRelation('coop_local.Dated')

    class Meta:
        abstract = True
        ordering = ['title']
        verbose_name = _(u'Organization')
        verbose_name_plural = _(u'Organizations')
        app_label = 'coop_local'

    def __unicode__(self):
        return unicode(self.title)

    def label(self):
        if self.pref_label == PREFLABEL.TITLE:
            return self.title
        elif self.pref_label == PREFLABEL.ACRO:
            return self.acronym

    if "coop_geo" in settings.INSTALLED_APPS:

        def has_location(self):
            return self.located.all().count() > 0
        has_location.boolean = True
        has_location.short_description = _(u'geo')

        def locations(self):
            from coop_geo.models import Location
            return Location.objects.filter(id__in=self.located.all().values_list('location_id', flat=True))

        def areas(self):
            from coop_geo.models import Area
            return Area.objects.filter(id__in=self.framed.all().values_list('location_id', flat=True))

        def to_geoJson(self):
            if self.pref_address and self.pref_address.point:
                return {
                   "type": "Feature",
                    "properties": {
                            "name": self.label(),
                            "popupContent": u"<h4>" + self.label() + u"</h4><p><a href='" + \
                            self.get_absolute_url() + u"'>" + self.label() + u"</a></p>"
                            },
                        "geometry": {
                            "type": "Point",
                            "coordinates": [self.pref_address.point.x, self.pref_address.point.y]
                            }
                        }

    def has_description(self):
        return self.description != None and len(self.description) > 20
    has_description.boolean = True
    has_description.short_description = _(u'desc.')

    def logo_list_display(self):
        try:
            if self.logo:
                thumb = default.backend.get_thumbnail(self.logo.file, ADMIN_THUMBS_SIZE)
                return '<img width="%s" src="%s" />' % (thumb.width, thumb.url)
            else:
                return _(u"No Image")
        except IOError:
            return _(u"No Image")

    logo_list_display.short_description = _(u"logo")
    logo_list_display.allow_tags = True

    #@models.permalink
    def get_absolute_url(self):
        return reverse('org_detail', args=[self.slug])

    def get_relations(self):
        relations = {}
        relmap = RELATIONS.REVERTED_CHOICES_CONST_DICT

        for rel in self.source.all():
            reltype = str('OUT_' + relmap[rel.reltype])  # me => others
            relations[reltype] = []
            relations[reltype].append(rel.target)
        for rel in self.target.all():
            reltype = str('IN_' + relmap[rel.reltype])  # others said this
            if reltype not in relations:
                relations[reltype] = []
            #if rel.confirmed:  # which one are confirmed by both parts
            relations[reltype].append(rel.source)
        return relations

    # def local_uri(self):
    #     return ('http://dev.credis.org:8000/org/' + self.slug + '/')

    def main_location(self):
        return self.located.all()[0].location

    def save(self, *args, **kwargs):
        # Set default values for preferred email, phone and postal address
        if self.pref_phone == None:
            phone_categories = [1, 2]
            fixes = self.contacts.filter(category__in=phone_categories)
            if fixes.count() == 1:
                self.pref_phone = fixes[0]
        if self.pref_email == None:
            orgmails = self.contacts.filter(category=8)
            if orgmails.count() == 1:
                self.pref_email = orgmails[0]
        if 'coop_geo' in settings.INSTALLED_APPS:
            if self.pref_address == None:
                locations = self.located.all()  # should we have a "main venue" ?
                if locations.count() == 1:
                    self.pref_address = locations[0].location

        # TODO move this to Contact model or do it in SQL

        # if self.email and self.email != '':
        #     import hashlib
        #     m = hashlib.sha1()
        #     m.update(self.email)
        #     self.email_sha1 = m.hexdigest()
        super(BaseOrganization, self).save(*args, **kwargs)

    def get_edit_url(self):
        return reverse('org_edit', args=[self.slug])

    def get_cancel_url(self):
        return reverse('org_edit_cancel', args=[self.slug])

    def _can_modify_organization(self, user):
        if user.is_authenticated():
            if user.is_superuser:
                return True
            elif user.person in self.members.all():
                return True
            else:
                return False

    def can_view_organization(self, user):
        # TODO use global privacy permissions on objects
        return True

    def can_edit_organization(self, user):
        return self._can_modify_organization(user)

    rdf_type = settings.NS.org.Organization
    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.legal.legalName, 'title'), 'single_reverse'),
        ('single_mapping', (settings.NS.ov.prefAcronym, 'acronym'), 'single_reverse'),
        ('single_mapping', (settings.NS.rdfs.comment, 'subtitle'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.description, 'description'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.mbox_sha1sum, 'email_sha1'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.homepage, 'web'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.birthday, 'birth'), 'single_reverse'),
        ('single_mapping', (settings.NS.vcard.tel, 'pref_phone'), 'single_reverse'),
        ('single_mapping', (settings.NS.vcard.mail, 'pref_email'), 'single_reverse'),
        ('single_mapping', (settings.NS.legal.registeredAddress, 'pref_address'), 'single_reverse'),
        ('single_mapping', (settings.NS.skos.note, 'notes'), 'single_reverse'),

        ('multi_mapping', (settings.NS.dct.subject, 'tags'), 'multi_reverse'),
        ('multi_mapping', (settings.NS.ess.hasContactMedium, 'contacts'), 'multi_reverse'),
        ('multi_mapping', (settings.NS.org.hasMember, 'members'), 'multi_reverse'),

        ('logo_mapping', (settings.NS.foaf.logo, 'logo'), 'logo_mapping_reverse'),
        ('prefLabel_mapping', (settings.NS.rdfs.label, 'pref_label'), 'prefLabel_mapping_reverse'),
        ('location_mapping', (settings.NS.locn.location, 'located'), 'location_mapping_reverse'),
        ('location_mapping', (settings.NS.ess.actionArea, 'framed'), 'location_mapping_reverse'),
        ('exchange_mapping', (settings.NS.gr.offers, settings.NS.gr.seeks), 'exchange_mapping_reverse'),

    ]


    def location_mapping(self, rdfPred, djF):
        values = map(lambda x: x.location, getattr(self, djF).all())
        return self.multi_mapping_base(values, rdfPred)

    def location_mapping_reverse(self, g, rdfPred, djField, lang=None):
        rdf_values = set(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        # Values contient des instances de Location. 
        # Il faut remonter soit a des Located soit a des AreaLink
        values = map(coop.models.StaticURIModel.toDjango, rdf_values)
        if djField == 'located':
            m = models.get_model('coop_geo', 'located')
            values = set(map(lambda x: m.objects.get(object_id=self.id, location=x), values))
        elif djField == 'framed':
            m = models.get_model('coop_geo', 'arealink')
            values = set(map(lambda x: m.objects.get(object_id=self.id, location=x), values))
        manager = getattr(self, djField)
        old_values = set(manager.all())
        remove = old_values.difference(values)
        add = values.difference(old_values)
        for v in remove:
            manager.remove(v)
        for v in add:
            manager.add(v)

    def logo_mapping(self, rdfPred, djF):
        logo = getattr(self, djF)
        if logo == None:
            return []
        else:
            try:
                rdfSubject = rdflib.term.URIRef(self.uri)
                rdfValue = rdflib.term.URIRef('http://' + str(Site.objects.get_current().domain) + logo.url)
                return [(rdfSubject, rdfPred, rdfValue)]
            except ValueError:
                return []

    def logo_mapping_reverse(self, g, rdfPred, djF):
        value = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        if len(value) == 1:
            value = value[0].toPython()
            scheme, host, path, query, fragment = urlsplit(value)
            sp = path.split('/')
            setattr(self, djF, "%s:%s" % (sp[len(sp) - 2], sp[len(sp) - 1]))
        else:
            pass

    def prefLabel_mapping(self, rdfPred, djF, lang=None):
        label = self.label()
        if label == None:
            return []
        else:
            subject_args = {}
            if lang:
                subject_args['lang'] = lang
            rdfValue = rdflib.term.Literal(unicode(label), **subject_args)
            return[(rdflib.term.URIRef(self.uri), rdfPred, rdfValue)]

    def prefLabel_mapping_reverse(self, g, rdfPred, djF, lang=None):
        value = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        title = list(g.objects(rdflib.term.URIRef(self.uri), settings.NS.dct.title))
        if value == title:
            setattr(self, 'pref_label', PREFLABEL.TITLE)
        else:
            setattr(self, 'pref_label', PREFLABEL.ACRO)

    def exchange_mapping(self, rdfOffer, rdfSeek, lang=None):
        from coop.exchange.models import EWAY
        values = models.get_model('coop_local', 'exchange').objects.filter(organization=self)
        rdfSubject = rdflib.term.URIRef(self.uri)
        result = []
        for value in values:
            if value.eway == EWAY.OFFER:
                result.append((rdfSubject, rdfOffer, rdflib.term.URIRef(value.uri)))
            else:
                result.append((rdfSubject, rdfSeek, rdflib.term.URIRef(value.uri)))
        return result

    def exchange_mapping_reverse(self, g, rdfOffer, rdfSeek, lang=None):
        from coop.exchange.models import EWAY
        values = list(g.objects(rdflib.term.URIRef(self.uri), rdfOffer))
        exchangeModel = models.get_model('coop_local', 'exchange')
        for value in values:
            ex = exchangeModel.objects.get(uri=str(value))
            ex.eway = EWAY.OFFER
            ex.organization = self
            ex.save()
        values = list(g.objects(rdflib.term.URIRef(self.uri), rdfSeek))
        for value in values:
            ex = exchangeModel.objects.get(uri=str(value))
            ex.eway = EWAY.NEED
            ex.organization = self
            ex.save()










# -*- coding:utf-8 -*-

from coop.models import StaticURIModel, URI_MODE
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.contrib.sites.models import Site
from django.conf import  settings
import rdflib
from rdflib import Graph, ConjunctiveGraph
import subhub
import logging
from urlparse import urlsplit
from django.db.models.loading import get_model
from django.contrib.contenttypes.models import ContentType
import threading
import time
import datetime
import coop_tag

if getattr(settings, 'SUBHUB_MAINTENANCE_AUTO', False):
    import django_rq


log = logging.getLogger('subhub.maintenance')



class LastDTProcess(object):
    _update = None

    @classmethod
    def get(cls):
        if not cls._update:
            cls._update = datetime.datetime.now() - datetime.timedelta(seconds=2)
        return cls._update - datetime.timedelta(seconds=2)

    @classmethod
    def update(cls):
        now = datetime.datetime.now()
        if not cls._update or now >= cls._update + datetime.timedelta(seconds=10):
            cls._update = now





# Workaround for rqworker limitation
# Be careful, this code is executed by the worker... in its MainThread (I guess
# the worker has a single thread)
# This implies some syncro question.
# For example, if the save is executed from the admin interface, it is
# excuted in its one thread. Thus the DistribustionTask is not created
# when this method is called. Passing the thread name in parameter, we can
# decide in which case we are and then add a small delay to be such that
# admin thread is finished
def letsCallDistributionTaskProcess(thName):
    time.sleep(2)   # wait until thread is finished
    #for dt in subhub.models.DistributionTask.objects.all():
        # log.debug(u'INQUUEU entry_id %s' % dt.entry_id)
    subhub.models.DistributionTask.objects.process(log=log)


# Listener tool
# The point here is to be careful with synchronization.
# subhub.publish will create a new DistributionTask object (if no DistributioTask
# exists for this instance); So the signals will be call again, that why
# the q.enqueue is done only when post_sae_callback is called with a DistributionTask
# as instance.
@receiver(post_save)
def post_save_callback(sender, instance, **kwargs):
    maintenance = getattr(settings, 'SUBHUB_MAINTENANCE_AUTO', False)
    # log.debug(u"Post save callback with sender %s and instance %s and AUTO %s" % (unicode(sender), unicode(instance), maintenance))

    if isinstance(instance, StaticURIModel):
        if instance.uri_mode == URI_MODE.IMPORTED:
            # log.debug(u"%s is imported. Nothing to publish, but subscription renew" % instance)
            instance.subscribeToUpdades()
            # TODO check if a subscription is done, either lets do it
        elif isinstance(instance, coop_tag.models.TagBase):
            # Subscription done if it does not exists, in other it is simply renew
            instance.subscribeToUpdades(host=settings.THESAURUS_HOST)
        else:
            feed_url = 'http://%s/feed/%s/' % (Site.objects.get_current().domain, sender.__name__.lower())
            try:
                subhub.publish([feed_url], instance.uri, False)
                #log.debug('publish done; Number of Dist task %s' % len(subhub.models.DistributionTask.objects.all()))
            except Exception, e:
                log.warning(u'Unable to publish %s for feed %s : %s' % (instance, feed_url, e))
    elif isinstance(instance, subhub.models.DistributionTask) and maintenance:
        try:
            nb = len(subhub.models.DistributionTask.objects.all())
            #log.debug("before call ENQUEUE, tasks %s" % nb)
            LastDTProcess.update()
            django_rq.enqueue(letsCallDistributionTaskProcess, threading.currentThread().name)
            #log.debug('after call ENQUEUE')
        except Exception, e:
            log.warning(u"%s" % e)
    elif isinstance(instance, subhub.models.SubscriptionTask) and maintenance:
        # call the maintenance
            try:
                log.info(u'Processing verification queue...')
                subhub.models.SubscriptionTask.objects.process(log=log)
            except subhub.utils.LockError, e:
                log.warning(u"%s" % e)
    else:
        pass


# TODO something has to be donne on deleting instance of StaticURIModel
@receiver(post_delete)
def post_delete_callback(sender, instance, **kwargs):
    if isinstance(instance, StaticURIModel):
        if instance.uri_mode == URI_MODE.IMPORTED:
            instance.unsubscribeToUpdades()


def listener(notification, **kwargs):
    ''' Process new content being provided by hub
    '''
    #log.debug("enter listener args %s" % kwargs)
    entry = notification.entries[0]
    uri = str(entry.summary)
    scheme, host, path, query, fragment = urlsplit(uri)
    sp = path.split('/')

   # We have to retreive the instance
    try:
        model = sp[len(sp) - 3]
        mType = ContentType.objects.get(model=model)
        obj = get_model(mType.app_label, model).objects.get(uri=uri)
    except Exception, e:
        obj = None
        log.error(u'%s' % e)

    #  build the rdf graph
    g = Graph()
    if model == 'tag':
        # This is a temporairy work around. data.economie-solidaire.fr is not
        # yet responding. TODO we have to deal with contexts as tags come thesaurus
        try:
            # TEMP hard coded
            thessEndPoint = 'http://localhost:8080/openrdf-workbench/repositories/thessRepository/query'
            graph = ConjunctiveGraph('SPARQLStore')
            graph.open(thessEndPoint, False)
            ctx = 'http://%s' % str(Site.objects.get_current().domain)
            ctx = rdflib.term.URIRef(ctx)
            localg = graph.get_context(ctx)
        except Exception, e:
            log.error(u'%s' % e)
        gen = localg.triples((rdflib.term.URIRef(uri), None, None))
        try:
            while True:
                g.add(gen.next())
        except:
            pass
        gen = localg.triples((None, None, rdflib.term.URIRef(uri)))
        try:
            while True:
                g.add(gen.next())
        except:
            pass
    else:
        try:
            g.parse(uri)
            #log.debug(u"Handle uri %s" % uri)
        except Exception, e:
            log.error(u"%s" % e)

    # update
    if obj:
        obj.import_rdf_data(g)





# -*- coding:utf-8 -*-

# Not used: to set this variable here, we need to import Site
PUSH_HUB = ''
SUBHUB_MAINTENANCE_AUTO = False

BASE_COOP_LOCAL_MODELS = [
    ('coop_cms', [
        u'Article',
        u'NavTree',
    ]),
    ('coop.link', [
        u'LinkProperty',
        u'Link',
    ]),
    ('coop_tag', [
        u'Tag',
        u'TaggedItem',
    ]),
    ('coop.agenda', [
        u'Calendar',
        u'Dated',
        u'Event',
        u'EventCategory',
        u'Occurrence',
    ]),
    ('coop.person', [
        u'Person',
        u'PersonCategory',
    ]),
    ('coop.org', [
        u'ContactMedium',
        u'Contact',
        u'Engagement',
        u'Role',
        u'RoleCategory',
        u'OrgRelationType',
        u'Relation',
        u'OrganizationCategory',
        u'Organization',
    ]),
    ('coop.exchange', [
        u'ExchangeMethod',
        u'Product',
        u'Exchange',
    ]),
    ('coop.mailing', [
        u'MailingList',
        u'Subscription',
    ]),
    ('coop.prefs', [
        u'SitePrefs',
    ]),
]


from rdflib import Namespace


class AttributeDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__


def transform_dict(dict):
    for key, value in dict.items():
        dict[key] = Namespace(value)
    return dict


RDF_NAMESPACES = {
 'ctag': u'http://commontag.org/ns#',
 'd2rq': u'http://www.wiwiss.fu-berlin.de/suhl/bizer/D2RQ/0.1#',
 'dct': u'http://purl.org/dc/terms/',
 'dce': u'http://purl.org/dc/elements/1.1/',
 'dcmi': u'http://purl.org/dc/dcmitype/',
 'ess':  u'http://ns.economie-solidaire.fr/ess#',
 'event': u'http://purl.org/NET/c4dm/event.owl#',
 'foaf': u'http://xmlns.com/foaf/0.1/',
 'geofr': u'http://rdf.insee.fr/geo/',
 'gr': u'http://purl.org/goodrelations/v1#',
 'legal': u'http://www.w3.org/ns/legal#',
 'locn': u'http://www.w3.org/ns/locn#',
 'opens': u'http://rdf.opensahara.com/type/geo/',
 'org': u'http://www.w3.org/ns/org#',
 'ov': u'http://open.vocab.org/terms/',
 'person': u'http://www.w3.org/ns/person#',
 'rdfs': u'http://www.w3.org/2000/01/rdf-schema#',
 'rdf': u'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
 'rss': u'http://purl.org/net/rss1.1#',
 'schema': u'http://schema.org/',
 'sioc': u'http://rdfs.org/sioc/ns#',
 'skos': u'http://www.w3.org/2004/02/skos/core#',
 'skosxl': u'http://www.w3.org/2008/05/skos-xl#',
 'vcal': u'http://www.w3.org/2002/12/cal/icaltzd#',
 'vcard': u'http://www.w3.org/2006/vcard/ns#',
 'xsd': 'http://www.w3.org/2001/XMLSchema#',
 }


# NS = AttributeDict( transform_dict(RDF_NAMESPACES).items() )





# -*- coding:utf-8 -*-

from coop.coop_bar_cfg import coop_perm
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from coop_bar.utils import make_link

can_edit_org = coop_perm('can_edit', ['organization'])
can_edit_org_category = coop_perm('can_edit', ['organizationcategory'])

# La methode du model doit s'appeller "can_edit_organization")


@can_edit_org
def org_edit_link(request, context):
    if not context.get('edit_mode'):
        org = context.get('organization')
        url = reverse('org_edit', args=[org.slug])
        return make_link(url, _(u'Edit organization'), 'fugue/store.png', classes=['icon'])


@can_edit_org_category
def org_category_edit_link(request, context):
    if not context.get('edit_mode'):
        org_category = context.get('org_category')
        url = reverse('org_category_edit', args=[org_category.slug])
        return make_link(url, _(u'Edit category'), 'fugue/store.png', classes=['icon'])


@can_edit_org
def org_view_link(request, context):
    if context.get('edit_mode'):
        org = context['organization']
        return make_link(org.get_cancel_url(), _(u'View'), 'fugue/eye.png',
            classes=['alert_on_click', 'icon', 'show-clean'])


@can_edit_org_category
def org_category_view_link(request, context):
    if context.get('edit_mode'):
        org_category = context['org_category']
        return make_link(org_category.get_cancel_url(), _(u'View'), 'fugue/eye.png',
            classes=['alert_on_click', 'icon', 'show-clean'])


@can_edit_org
@can_edit_org_category
def org_save(request, context):
    if context.get('edit_mode'):
        #No link, will be managed by catching the js click event
        return make_link('', _(u'Save'), 'fugue/tick.png', id="coopbar_save",
            classes=['show-dirty', 'icon'])


@can_edit_org
def org_edit_cancel_link(request, context):
    if context.get('edit_mode'):
        org = context['organization']
        return make_link(org.get_cancel_url(), _(u'Cancel'), 'fugue/cross.png',
            classes=['alert_on_click', 'icon', 'show-dirty'])


@can_edit_org_category
def org_category_edit_cancel_link(request, context):
    if context.get('edit_mode'):
        org_category = context['org_category']
        return make_link(org_category.get_cancel_url(), _(u'Cancel'), 'fugue/cross.png',
            classes=['alert_on_click', 'icon', 'show-dirty'])


def org_admin_link(request, context):
    if request and request.user.is_staff and 'organization' in context:
        if not context.get('edit_mode'):
            org = context['organization']
            view_name = 'admin:%s_%s_change' % ('coop_local', 'organization')
            return make_link(reverse(view_name, args=[org.id]), _(u"Edit via admin"), 'fugue/application-form.png',
                classes=['icon', 'alert_on_click'])



#!/usr/bin/env python
from pprint import pprint, pformat
from cStringIO import StringIO
from datetime import datetime, timedelta, date, time

from django.test import TestCase
from django.contrib.auth.models import User
from django.core.management import call_command

from coop.agenda import utils
from coop_local.models import Event, Occurrence

expected_table_1 = """\
| 15:00 |          |          |          |          |          |
| 15:15 | zelda    |          |          |          |          |
| 15:30 | zelda    | alpha    |          |          |          |
| 15:45 |          | alpha    |          |          |          |
| 16:00 | bravo    | alpha    | foxtrot  |          |          |
| 16:15 | bravo    | alpha    | foxtrot  | charlie  |          |
| 16:30 | bravo    | alpha    | foxtrot  | charlie  | delta    |
| 16:45 |          | alpha    |          | charlie  | delta    |
| 17:00 |          | alpha    |          |          | delta    |
| 17:15 | echo     | alpha    |          |          |          |
| 17:30 | echo     | alpha    |          |          |          |
| 17:45 | echo     |          |          |          |          |
| 18:00 |          |          |          |          |          |
"""

expected_table_2 = """\
| 15:30 | zelda    | alpha    |          |          |          |
| 15:45 |          | alpha    |          |          |          |
| 16:00 | bravo    | alpha    | foxtrot  |          |          |
| 16:15 | bravo    | alpha    | foxtrot  | charlie  |          |
| 16:30 | bravo    | alpha    | foxtrot  | charlie  | delta    |
| 16:45 |          | alpha    |          | charlie  | delta    |
| 17:00 |          | alpha    |          |          | delta    |
| 17:15 | echo     | alpha    |          |          |          |
| 17:30 | echo     | alpha    |          |          |          |
"""

expected_table_3 = """\
| 16:00 | alpha    | bravo    | foxtrot  |          |          |
| 16:15 | alpha    | bravo    | foxtrot  | charlie  |          |
| 16:30 | alpha    | bravo    | foxtrot  | charlie  | delta    |
| 16:45 | alpha    |          |          | charlie  | delta    |
| 17:00 | alpha    |          |          |          | delta    |
| 17:15 | alpha    | echo     |          |          |          |
| 17:30 | alpha    | echo     |          |          |          |
"""

expected_table_4 = """\
| 18:00 |          |          |          |          |
| 18:15 |          |          |          |          |
| 18:30 |          |          |          |          |
| 18:45 |          |          |          |          |
| 19:00 |          |          |          |          |
| 19:15 |          |          |          |          |
| 19:30 |          |          |          |          |
"""

expected_table_5 = """\
| 16:30 | alpha    | bravo    | foxtrot  | charlie  | delta    |
"""


class TableTest(TestCase):
    fixtures = ['agenda_test']

    def setUp(self):
        self._dt = dt = datetime(2008,12,11)

    def table_as_string(self, table):
        timefmt = '| %-5s'
        cellfmt = '| %-8s'
        out = StringIO()
        for tm, cells in table:
            print >> out, timefmt % tm.strftime('%H:%M'),
            for cell in cells:
                if cell:
                    print >> out, cellfmt % cell.event.title,
                else:
                    print >> out, cellfmt % '',
            print >> out, '|'

        return out.getvalue()

    def _do_test(self, start, end, expect):
        import pdb
        start = time(*start)
        dtstart = datetime.combine(self._dt, start)
        etd = datetime.combine(self._dt, time(*end)) - dtstart

        # pdb.set_trace()
        table = utils.create_timeslot_table(self._dt, start_time=start, end_time_delta=etd)

        actual = self.table_as_string(table)
        out = 'Expecting:\n%s\nActual:\n%s' % (expect, actual)
        print out
        self.assertEqual(actual, expect, out)

    def test_slot_table_1(self):
        self._do_test((15,0), (18,0), expected_table_1)

    def test_slot_table_2(self):
        self._do_test((15,30), (17,30), expected_table_2)

    def test_slot_table_3(self):
        self._do_test((16,0), (17,30), expected_table_3)

    def test_slot_table_4(self):
        self._do_test((18,0), (19,30), expected_table_4)

    def test_slot_table_5(self):
        self._do_test((16,30), (16,30), expected_table_5)


class NewEventFormTest(TestCase):
    fixtures = ['agenda_test']

    def test_new_event_simple(self):
        from coop.agenda.forms import EventForm, MultipleOccurrenceForm

        data = dict(
            title='QWERTY',
            event_type='1',
            day='2008-12-11',
            start_time_delta='28800',
            end_time_delta='29700',
            year_month_ordinal_day='2',
            month_ordinal_day='2',
            holidays='skip',
            year_month_ordinal='1',
            month_option='each',
            repeats='count',
            freq='2',
            occurences='2',
            month_ordinal='1'
        )

        evt_form = EventForm(data)
        occ_form = MultipleOccurrenceForm(data)
        self.assertTrue(evt_form.is_valid(), evt_form.errors.as_text())
        self.assertTrue(occ_form.is_valid(), occ_form.errors.as_text())

        self.assertEqual(
            occ_form.cleaned_data['start_time'],
            datetime(2008, 12, 11, 8),
            'Bad start_time: %s' % pformat(occ_form.cleaned_data)
        )


def doc_tests():
    """
        >>> from dateutil import rrule
        >>> from datetime import datetime
        >>> from coop_local.models import EventCategory
        >>> evt_types = [EventCategory.objects.create(abbr=l.lower(),label=l) for l in ['Foo', 'Bar', 'Baz']]
        >>> evt_types
        [<EventCategory: Foo>, <EventCategory: Bar>, <EventCategory: Baz>]
        >>> e = Event.objects.create(title='Hello, world', description='Happy New Year', event_type=evt_types[0])
        >>> e
        <Event: Hello, world>
        >>> e.add_occurrences(datetime(2008,1,1), datetime(2008,1,1,1), freq=rrule.YEARLY, count=7)
        >>> e.occurrence_set.all()
        [<Occurrence: Hello, world: 2008-01-01T00:00:00>, <Occurrence: Hello, world: 2009-01-01T00:00:00>, <Occurrence: Hello, world: 2010-01-01T00:00:00>, <Occurrence: Hello, world: 2011-01-01T00:00:00>, <Occurrence: Hello, world: 2012-01-01T00:00:00>, <Occurrence: Hello, world: 2013-01-01T00:00:00>, <Occurrence: Hello, world: 2014-01-01T00:00:00>]
        >>> e = create_event('Bicycle repairman', event_type=evt_types[2])
        >>> e.occurrence_set.count()
        1
        >>> e = create_event(
        ...     'Something completely different',
        ...     event_type=('abbr', 'Abbreviation'),
        ...     start_time=datetime(2008,12,1, 12),
        ...     freq=rrule.WEEKLY,
        ...     byweekday=(rrule.TU, rrule.TH),
        ...     until=datetime(2008,12,31)
        ... )
        >>> for o in e.occurrence_set.all():
        ...     print o.start_time, o.end_time
        ...
        2008-12-02 12:00:00 2008-12-02 13:00:00
        2008-12-04 12:00:00 2008-12-04 13:00:00
        2008-12-09 12:00:00 2008-12-09 13:00:00
        2008-12-11 12:00:00 2008-12-11 13:00:00
        2008-12-16 12:00:00 2008-12-16 13:00:00
        2008-12-18 12:00:00 2008-12-18 13:00:00
        2008-12-23 12:00:00 2008-12-23 13:00:00
        2008-12-25 12:00:00 2008-12-25 13:00:00
        2008-12-30 12:00:00 2008-12-30 13:00:00
    """
    pass


# -*- coding:utf-8 -*-

from django.conf import settings
from coop.link.models import BaseLinkProperty, BaseLink
from coop.article.models import CoopArticle, CoopNavTree
from coop.person.models import BasePerson, BasePersonCategory
from coop.exchange.models import BaseExchange, BaseProduct, BaseExchangeMethod
from coop.org.models import (BaseOrganizationCategory, BaseOrganization,
                             BaseRelation, BaseEngagement, BaseRole, BaseOrgRelationType,
                             BaseContact, BaseRoleCategory, BaseContactMedium)
from coop.prefs.models import BaseSitePrefs

# ---- person


class LinkProperty(BaseLinkProperty):
    pass


class Link(BaseLink):
    pass


# ----- CMS

class Article(CoopArticle):
    pass


class NavTree(CoopNavTree):
    pass

if 'coop_tag' in settings.INSTALLED_APPS:
    from coop.tag.models import CoopTag, CoopTaggedItem

    # ----- Tag

    class Tag(CoopTag):
        pass

    class TaggedItem(CoopTaggedItem):
        pass


# ---- person


class PersonCategory(BasePersonCategory):
    pass


class Person(BasePerson):
    pass


# ----- org


class ContactMedium(BaseContactMedium):
    pass


class Contact(BaseContact):
    pass


class RoleCategory(BaseRoleCategory):
    pass


class Role(BaseRole):
    pass


class Relation(BaseRelation):
    pass


class Engagement(BaseEngagement):
    pass


class OrgRelationType(BaseOrgRelationType):
    pass


class OrganizationCategory(BaseOrganizationCategory):
    pass


class Organization(BaseOrganization):
    pass


# ------ exchange


class Exchange(BaseExchange):
    pass


class ExchangeMethod(BaseExchangeMethod):
    pass


class Product(BaseProduct):
    pass


# ----- mailing

if 'coop.mailing' in settings.INSTALLED_APPS:
    from coop.mailing.models import BaseSubscription, BaseMailingList
    
    class MailingList(BaseMailingList):
        pass

    class Subscription(BaseSubscription):
        pass


# ----- agenda

if 'coop.agenda' in settings.INSTALLED_APPS:
    from coop.agenda.models import BaseCalendar, BaseEvent, BaseEventCategory, \
            BaseOccurrence, BaseDated

    class Calendar(BaseCalendar):
        pass

    class Event(BaseEvent):
        pass

    class EventCategory(BaseEventCategory):
        pass

    class Occurrence(BaseOccurrence):
        pass

    class Dated(BaseDated):
        pass


# ------ global site preferences


class SitePrefs(BaseSitePrefs):
    pass



"""
This file was generated with the custommenu management command, it contains
the classes for the admin menu, you can customize this class as you want.

To activate your custom menu add the following to your settings.py::
    ADMIN_TOOLS_MENU = 'devcoop.menu.CustomMenu'
"""

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from admin_tools.menu import items, Menu


class CustomMenu(Menu):
    """
    This is a base Menu for django-coop admin site.

    You can :
    - override the whole thing using the ADMIN_TOOLS_MENU setting
    - just add your own custom menus by passing a list of dicts in coop_local/local_settings.py :

    ADMINTOOLS_CUSTOM_MENUS = [
            {'title': 'My Own Menu',
             'icon': 'icon-tint icon-white',  # a bootstrap icon, see http://twitter.github.com/bootstrap/base-css.html#images
             'children': [
                            ('First sub-menu', '/admin/coop_local/my_custom_model/'),
                            ('Second sub-menu', '/admin/coop_local/another_custom_model/'),
                        ]}
                ]

    """
    def __init__(self, **kwargs):
        Menu.__init__(self, **kwargs)
        self.children = [
            #items.MenuItem(_('Dashboard'), reverse('admin:index')),
            #items.Bookmarks('Favoris'),

            items.MenuItem(_('Navigation tree'), '/admin/coop_local/navtree/1/', icon='icon-list-alt icon-white'),

            items.MenuItem(_('Articles'), '/admin/coop_local/article/', icon='icon-pencil icon-white'),

            items.MenuItem(_('CMS'), '#', icon='icon-cog icon-white',
                children=[

                    items.MenuItem(_('Content'), '#', icon='icon-file', children=[
                        items.MenuItem(_('Article categories'), '/admin/coop_cms/articlecategory/'),
                        items.MenuItem(_('Documents'), '/admin/coop_cms/document/'),
                        items.MenuItem(_('Images'), '/admin/coop_cms/image/'),
                        items.MenuItem(_('Newsletters'), '/admin/coop_cms/newsletter/'),
                        items.MenuItem(_('Comments'), '/admin/comments/comment/'),
                        items.MenuItem(_('Forms'), '/admin/forms/form/'),
                        items.MenuItem(_('Preferences'), '/admin/coop_local/siteprefs/'),

                        ]),

                    # RSS Sync menu gets inserted here if installed (see above)

                    items.MenuItem(_('Tags'), '#', icon='icon-tags', children=[
                        items.MenuItem(_('Tags'), '/admin/coop_local/tag/'),
                        #items.MenuItem(_('Tag categories'), '/admin/coop_tag/tagcategory/'),
                        items.MenuItem(_('Tag trees'), '/admin/coop_local/navtree/'),
                        ]),

                    items.MenuItem(_('Django'), '#', icon='icon-coop icon-django', children=[
                        items.MenuItem(_('Users'), '/admin/auth/user/'),
                        items.MenuItem(_('Sites'), '/admin/sites/site/'),
                        ]),
                ]
            ),


            # Agenda menu inserted here if coop.agenda installed

            items.MenuItem(_('Network'), '#', icon='icon-coop icon-group icon-white',
                children=[

                    items.MenuItem(_('Directory'), '#', icon='icon-home', children=[
                        items.MenuItem(_('Organizations'), '/admin/coop_local/organization/'),
                        items.MenuItem(_('Persons'), '/admin/coop_local/person/'),
                        items.MenuItem(_('Organization categories'), '/admin/coop_local/organizationcategory/'),
                        items.MenuItem(_('Person categories'), '/admin/coop_local/personcategory/'),
                        items.MenuItem(_('Roles'), '/admin/coop_local/role/'),
                        ]),

                    items.MenuItem(_('Exchanges'), '#', icon='icon-random', children=[
                        items.MenuItem(_('Exchanges'), '/admin/coop_local/exchange/'),
                        items.MenuItem(_('Exchange methods'), '/admin/coop_local/exchangemethod/'),
                        ]),

                    items.MenuItem(_('Cartography'), '#', icon='icon-map-marker', children=[
                        items.MenuItem(_('Locations'), '/admin/coop_geo/location/'),
                        items.MenuItem(_('Areas'), '/admin/coop_geo/area/'),
                        items.MenuItem(_('Location categories'), '/admin/coop_geo/locationcategory/'),
                        # create my map !
                        ]),

                    items.MenuItem(_('RDF settings'), '#', icon='icon-coop icon-rdf', children=[
                        items.MenuItem(_('URI redirection'), '/admin/uriredirect/'),
                        # webid
                        ]),


                ]
            ),



        ]

        if 'coop_cms.apps.rss_sync' in settings.INSTALLED_APPS:
            self.children[2].children.insert(1,

                    items.MenuItem(_('RSS'), '#', icon='icon-coop icon-rss', children=[
                        items.MenuItem(_('RSS items'), '/admin/rss_sync/rssitem/'),
                        items.MenuItem(_('RSS sources'), '/admin/rss_sync/rsssource/'),
                        ])
                    )

        if 'coop.agenda' in settings.INSTALLED_APPS:
            self.children.insert(2,
                items.MenuItem(_('Agenda'), '#', icon='icon-calendar icon-white',
                    children=[
                        items.MenuItem(_('Events'), '/admin/coop_local/event/'),
                        items.MenuItem(_('Calendar'), '/admin/coop_local/calendar/'),
                        items.MenuItem(_('Event categories'), '/admin/coop_local/eventcategory/'),
                    ])
                )

        if settings.ADMINTOOLS_CUSTOM_MENUS:
            idx = len(self.children)
            for menu in settings.ADMINTOOLS_CUSTOM_MENUS:
                self.children.insert(idx, items.MenuItem(
                    menu['title'], '#',
                    icon=menu['icon'],
                    children=[items.MenuItem(x[0], x[1]) for x in menu['children']]
                    ))
                idx += 1

    def init_with_context(self, context):
        """
        Use this method if you need to access the request context.
        """
        return super(CustomMenu, self).init_with_context(context)



# -*- coding:utf-8 -*-

from datetime import datetime

from django.utils.translation import ugettext_lazy as _
from django.db import models
from django_extensions.db import fields as exfields
from dateutil import rrule
from django.db.models.loading import get_model
from coop.models import URIModel
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.conf import settings
import rdflib
from django.template.defaultfilters import date as _date
from datetime import datetime


class BaseCalendar(URIModel):
    title = models.CharField(_('title'), blank=True, max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    slug = exfields.AutoSlugField(populate_from='title')

    class Meta:
        verbose_name = _('calendar')
        verbose_name_plural = _('calendars')
        abstract = True
        app_label = 'coop_local'

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('agenda-default', args=[self.slug])

    rdf_type = settings.NS.vcal.Vcalendar
    base_mapping = [
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.title, 'title'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.description, 'description'), 'single_reverse'),

        ('component_mapping', (settings.NS.vcal.component, ''), 'component_mapping_reverse'),

    ]

    def component_mapping(self, rdfPred, djF, lang=None):
        events = models.get_model('coop_local', 'event').objects.filter(calendar=self)
        res = []
        for e in events:
            res.append((rdflib.URIRef(self.uri), rdfPred, rdflib.URIRef(e.uri)))
        return res

    def component_mapping_reverse(self, g, rdfPred, rdfEnd, lang=None):
        values = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        m = models.get_model('coop_local', 'event')
        for v in values:
            event, created = m.objects.get_or_create(uri=str(v))
            event.calendar = self
            event.save()


class BaseEventCategory(models.Model):
    """
    Simple ``Event`` classification.
    """
    # abbr = models.CharField(_('abbreviation'), max_length=4, unique=True)  #varennes
    label = models.CharField(_('label'), max_length=50)
    slug = exfields.AutoSlugField(populate_from='label')

    class Meta:
        verbose_name = _('event category')
        verbose_name_plural = _('event categories')
        abstract = True
        app_label = 'coop_local'

    def __unicode__(self):
        return self.label

    @models.permalink
    def get_absolute_url(self):
        return ('agenda-event-category', [str(self.slug)])

    def events(self):
        return self.event_set.all()
        

class BaseEvent(URIModel):
    """
    Container model for general metadata and associated ``Occurrence`` entries.
    """
    title = models.CharField(_('title'), max_length=120)
    description = models.TextField(_(u'description'), blank=True)
    slug = exfields.AutoSlugField(populate_from='title')
    event_type = models.ForeignKey('coop_local.EventCategory', verbose_name=_('event type'))  # TODO rename category !!!
    calendar = models.ForeignKey('coop_local.Calendar', verbose_name=_('calendar'))

    # Linking to local objects
    organization = models.ForeignKey('coop_local.Organization', null=True, blank=True, verbose_name=_('organization'), related_name=_('publisher organization'))
    organizations = models.ManyToManyField('coop_local.Organization', null=True, blank=True, verbose_name=_('organizations'), related_name=_('other organizations'))

    person = models.ForeignKey('coop_local.Person', null=True, blank=True, verbose_name=_('author'))
    if "coop_geo" in settings.INSTALLED_APPS:
        location = models.ForeignKey('coop_geo.Location', null=True, blank=True, verbose_name=_('location'))
        remote_location_uri = models.URLField(_('remote location URI'), blank=True, max_length=255)
        remote_location_label = models.CharField(_(u'remote location label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the location record is not available locally'))

    # Linking to remote objects
    remote_person_uri = models.URLField(_('remote person URI'), blank=True, max_length=255)
    remote_person_label = models.CharField(_(u'remote person label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the person record is not available locally'))
    remote_organization_uri = models.URLField(_('remote organization URI'), blank=True, max_length=255)
    remote_organization_label = models.CharField(_(u'remote organization label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the organization record is not available locally'))

    class Meta:
        verbose_name = _('event')
        verbose_name_plural = _('events')
        ordering = ('title', )
        abstract = True
        app_label = 'coop_local'

    def label(self):
        return self.title

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('agenda-event', [str(self.id)])

    @property
    def linked_articles(self):
        linked_items = self.dated_set.all()
        if linked_items:
            result = []
            for item in linked_items:
                result.append(item.content_object)
            return result    
        else:
            return None

    def add_occurrences(self, start_time, end_time, **rrule_params):
        """
        Add one or more occurences to the event using a comparable API to
        ``dateutil.rrule``.

        If ``rrule_params`` does not contain a ``freq``, one will be defaulted
        to ``rrule.DAILY``.

        Because ``rrule.rrule`` returns an iterator that can essentially be
        unbounded, we need to slightly alter the expected behavior here in order
        to enforce a finite number of occurrence creation.

        If both ``count`` and ``until`` entries are missing from ``rrule_params``,
        only a single ``Occurrence`` instance will be created using the exact
        ``start_time`` and ``end_time`` values.
        """
        rrule_params.setdefault('freq', rrule.DAILY)

        if 'count' not in rrule_params and 'until' not in rrule_params:
            self.occurrence_set.create(start_time=start_time, end_time=end_time)
        else:
            delta = end_time - start_time
            for ev in rrule.rrule(dtstart=start_time, **rrule_params):
                self.occurrence_set.create(start_time=ev, end_time=ev + delta)

    def upcoming_occurrences(self):
        """
        Return all occurrences that are set to start on or after the current
        time.
        """
        return self.occurrence_set.filter(start_time__gte=datetime.now())

    def next_occurrence(self):
        """
        Return the single occurrence set to start on or after the current time
        if available, otherwise ``None``.
        """
        upcoming = self.upcoming_occurrences()
        return upcoming and upcoming[0] or None

    def daily_occurrences(self, dt=None):
        """
        Convenience method wrapping ``Occurrence.objects.daily_occurrences``.
        """
        return get_model('coop_local', 'Occurrence').objects.daily_occurrences(dt=dt, event=self)


    # RDF stuffs
    # Attention il faut tenir compte des occurences.... ca complique un peu


    rdf_type = settings.NS.vcal.Vevent
    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.vcal.summary, 'title'), 'single_reverse'),
        ('single_mapping', (settings.NS.vcal.description, 'description'), 'single_reverse'),
 
        ('local_or_remote_mapping', (settings.NS.vcal.contact, 'person'), 'local_or_remote_reverse'),
        ('local_or_remote_mapping', (settings.NS.vcal.organizer, 'organization'), 'local_or_remote_reverse'),
        ('local_or_remote_mapping', (settings.NS.locn.location, 'location'), 'local_or_remote_reverse'),

        ('multi_mapping', (settings.NS.dct.subject, 'tags'), 'multi_reverse'),
        ('multi_mapping', (settings.NS.vcal.attendee, 'organizations'), 'multi_reverse'),

        ('article_mapping', (settings.NS.dct.relation, 'linked_articles'), 'article_mapping_reverse'),
        ('category_mapping', (settings.NS.vcal.categories, 'event_type'), 'category_mapping_reverse'),
        ('occurence_mapping', (settings.NS.vcal.dtstart, settings.NS.vcal.dtend), 'occurence_mapping_reverse'),
    ]


    def article_mapping(self, rdfPred, djF, datatype=None, lang=None):
        return self.multi_mapping(rdfPred, djF, datatype, lang)

    def article_mapping_reverse(self, g, rdfPred, djField, lang=None):
        articles = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        model_article = models.get_model('coop_local', 'article')
        model_dated = models.get_model('coop_local', 'dated')

        for a in articles:
            ar = model_article.get_or_create_from_rdf(uri=str(a))
            new_dated = model_dated(content_object=ar, event=self)
            new_dated.save()


    def category_mapping(self, rdfPred, djF, lang=None):
        value = getattr(self, djF).label
        return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(value))]

    def category_mapping_reverse(self, g, rdfPred, djField, lang=None):
        values = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        m = models.get_model('coop_local', 'eventcategory')
        if len(values) == 1:
            value = values[0]
            try:
                djValue = m.objects.get(label=value)
                setattr(self, djField, djValue)
            except m.DoesNotExist:
                pass

    def occurence_mapping(self, rdfStart, rdfEnd, lang=None):
        m = models.get_model('coop_local', 'occurrence')
        occurrences = m.objects.filter(event=self)
        if len(occurrences) == 1:  # ouf rien à faire
            occurrence = occurrences[0]
            return [(rdflib.term.URIRef(self.uri), rdfStart, rdflib.term.Literal(occurrence.start_time)),
                    (rdflib.term.URIRef(self.uri), rdfEnd, rdflib.term.Literal(occurrence.end_time))]
        elif len(occurrences) == 0:
            return []
        else:  # On gère les occurrences à minima et en trichant un peu on modifie l'uri stockée
            i = 0
            res = []
            for occurrence in occurrences:
                i += 1
                uri = self.uri + '#%s' % i
                res.append((rdflib.term.URIRef(uri), settings.NS.rdf.type, self.rdf_type))
                res.append((rdflib.term.URIRef(uri), settings.NS.dct.modified, self.modified))
                res.append((rdflib.term.URIRef(uri), rdfStart, rdflib.term.Literal(occurrence.start_time)))
                res.append((rdflib.term.URIRef(uri), rdfEnd, rdflib.term.Literal(occurrence.end_time)))
            return res  # en attendant meiux

    def occurence_mapping_reverse(self, g, rdfStart, rdfEnd, lang=None):
        start = list(g.objects(rdflib.term.URIRef(self.uri), rdfStart))
        m = models.get_model('coop_local', 'occurrence')

        if len(start) == 1:
            end = list(g.objects(rdflib.term.URIRef(self.uri), rdfEnd))
            duration = list(g.objects(rdflib.term.URIRef(self.uri), settings.NS.vcal.duration))
            start = start[0].toPython()
            if len(end) == 1:
                occur, created = m.objects.get_or_create(start_time=start, end_time=end[0].toPython(), event=self)
            elif len(duration) == 1:
                end = start + duration[0].toPython()
                occur, created = m.objects.get_or_create(start_time=start, end_time=end, event=self)
        elif len(start) == 0:
            # Il faut chercher les uri de la forme self.uri#i avec i = 1,2,3,....
            i = 0
            again = True
            while again:
                i += 1
                uri = self.uri + "#%s" % i
                start = list(g.objects(rdflib.term.URIRef(uri), rdfStart))
                again = not start == []
                if len(start) == 1:
                    end = list(g.objects(rdflib.term.URIRef(uri), rdfEnd))
                    duration = list(g.objects(rdflib.term.URIRef(uri), settings.NS.vcal.duration))
                    start = start[0].toPython()
                    if len(end) == 1:
                        occur, created = m.objects.get_or_create(start_time=start, end_time=end[0].toPython(), event=self)
                    elif len(duration) == 1:
                        end = start + duration[0].toPython()
                        occur, created = m.objects.get_or_create(start_time=start, end_time=end, event=self)
                else:
                    pass
        else:
            pass


class BaseDated(models.Model):
    # things which are linked to an event
    event = models.ForeignKey('coop_local.Event', null=True, blank=True,
                                 verbose_name=_(u"event"))
    # generic key to other objects
    content_type = models.ForeignKey(ContentType, blank=True, null=True, default=None)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    def __unicode__(self):
        return unicode(self.content_object) + u" @ " + unicode(self.event)

    class Meta:
        abstract = True
        verbose_name = _(u'Dated item')
        verbose_name_plural = _(u'Dated items')
        app_label = 'coop_local'


class OccurrenceManager(models.Manager):
    use_for_related_fields = True

    def daily_occurrences(self, dt=None, event=None):
        """
        Returns a queryset of for instances that have any overlap with a
        particular day.

        * ``dt`` may be either a datetime.datetime, datetime.date object, or
          ``None``. If ``None``, default to the current day.

        * ``event`` can be an ``Event`` instance for further filtering.
        """
        dt = dt or datetime.now()
        start = datetime(dt.year, dt.month, dt.day)
        end = start.replace(hour=23, minute=59, second=59)
        qs = self.filter(
            models.Q(
                start_time__gte=start,
                start_time__lte=end,
            ) |
            models.Q(
                end_time__gte=start,
                end_time__lte=end,
            ) |
            models.Q(
                start_time__lt=start,
                end_time__gt=end
            )
        )
        if event:
            return qs.filter(event=event)
        else:
            return qs


class BaseOccurrence(models.Model):
    """
    Represents the start end time for a specific occurrence of a master ``Event``
    object.
    """
    start_time = models.DateTimeField(_('start time'))
    end_time = models.DateTimeField(_('end time'))
    event = models.ForeignKey('coop_local.Event', verbose_name=_('event'), editable=False)

    objects = OccurrenceManager()

    class Meta:
        verbose_name = _('occurrence')
        verbose_name_plural = _('occurrences')
        ordering = ('start_time', 'end_time')
        abstract = True
        app_label = 'coop_local'

    def __unicode__(self):
        return _date(self.start_time, _("l j F Y"))

    @models.permalink
    def get_absolute_url(self):
        return ('agenda-occurrence', [str(self.event.id), str(self.id)])

    def __cmp__(self, other):
        return cmp(self.start_time, other.start_time)

    @property
    def title(self):
        return self.event.title

    @property
    def event_type(self):
        return self.event.event_type


def create_event(title, event_type, description='', start_time=None,
        end_time=None, note=None, **rrule_params):
    """
    Convenience function to create an ``Event``, optionally create an
    ``EventCategory``, and associated ``Occurrence``s. ``Occurrence`` creation
    rules match those for ``Event.add_occurrences``.

    Returns the newly created ``Event`` instance.

    Parameters

    ``event_type``
        can be either an ``EventCategory`` object or 2-tuple of ``(abbreviation,label)``,
        from which an ``EventCategory`` is either created or retrieved.

    ``start_time``
        will default to the current hour if ``None``

    ``end_time``
        will default to ``start_time`` plus agenda_settings.DEFAULT_OCCURRENCE_DURATION
        hour if ``None``

    ``freq``, ``count``, ``rrule_params``
        follow the ``dateutils`` API (see http://labix.org/python-dateutil)
    """
    from coop.agenda.conf import settings as agenda_settings

    if isinstance(event_type, tuple):
        event_type, created = get_model('coop_local', 'EventCategory').objects.get_or_create(
            abbr=event_type[0],
        )
        if created:
            event_type.label = event_type[1]
            event_type.save()

    event = get_model('coop_local', 'Event').objects.create(
        title=title,
        description=description,
        event_type=event_type
    )

    start_time = start_time or datetime.now().replace(
        minute=0,
        second=0,
        microsecond=0
    )

    end_time = end_time or start_time + agenda_settings.DEFAULT_OCCURRENCE_DURATION
    event.add_occurrences(start_time, end_time, **rrule_params)
    return event



# -*- coding:utf-8 -*-
from django.contrib import admin
from coop_local.models import MailingList

# from django.conf import settings
# from django.utils.translation import ugettext_lazy as _
# from django.contrib import admin, messages
# from coop.mailing.forms import MailingListAdminForm
# from coop_local.models import MailingList
# # from coop.mailing.models import Parameter
# from coop.mailing import soap 



# class ParameterInline(admin.StackedInline):
#     model = Parameter


# class MailingListAdmin(admin.ModelAdmin):
#     form = MailingListAdminForm
#     # fields = ['name', 'subject', 'template', 'description', 'topics']
#     # inlines = [
#     #     ParameterInline,
#     # ]

#     # def save_formset(self, request, form, formset, change):
#     #     import pdb
#     #     pdb.set_trace()
#     #     if not change:
#     #         obj = form.save(commit=False)
#     #         if obj.template == 8 :
#     #             subject = '%s%shttp://%s/sympa_remote_list/%s' % (obj.subject, settings.SYMPA_SOAP['PARAMETER_SEPARATOR'],Site.objects.get_current(), obj.name)
#     #         else:
#     #             subject = obj.subject
#     #         subject = len(formset.cleaned_data) > 0 and '%s%s%s' % (obj.subject, settings.SYMPA_SOAP['PARAMETER_SEPARATOR'], settings.SYMPA_SOAP['PARAMETER_SEPARATOR'].join([data['value'] for data in formset.cleaned_data if data.has_key('value')])) or obj.subject
#     #         result = soap.create_list(obj.name, subject, obj.templateName, obj.description)
#     #         if result == 1:
#     #             obj.save()
#     #             formset.save()
#     #         else:
#     #             obj.delete()
#     #             messages.error(request, _(u"Cannot add the list : %s" % result))


#     def delete_model(self, request, obj):
#         result = 1
#         if soap.exists(obj.name):
#             result = soap.close_list(obj.name)

#         if result == 1 or result == 'list allready closed':
#             obj.delete()
#         else:
#             messages.error(request, _(u"Cannot close the list : %s" % result))

admin.site.register(MailingList)



# -*- coding:utf-8 -*-

from django.conf import settings
from coop.models import StaticURIModel, URIModel
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
import rdflib


# ----------- Customizing coop-cms Article for more coop intergration

if "coop_cms" in settings.INSTALLED_APPS:

    from coop_cms.models import BaseArticle, BaseNavTree

    class CoopArticle(BaseArticle, StaticURIModel):

        organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                                verbose_name=_('publisher'), related_name='articles')
        person = models.ForeignKey('coop_local.Person', blank=True, null=True,
                                    verbose_name=_(u'author'), related_name='articles')
        # Linking to remote objects
        remote_person_uri = models.URLField(_(u'remote person URI'), blank=True, max_length=255)
        remote_person_label = models.CharField(_(u'remote person label'),
                                                    max_length=250, blank=True, null=True,
                                                    help_text=_(u'fill this only if the person record is not available locally'))
        remote_organization_uri = models.URLField(_(u'remote organization URI'), blank=True, max_length=255)
        remote_organization_label = models.CharField(_(u'remote organization label'),
                                                    max_length=250, blank=True, null=True,
                                                    help_text=_(u'fill this only if the organization record is not available locally'))

        isSection = models.BooleanField(_(u'is section'), default=False, 
                                        help_text=_(u"Afficher les articles de niveau inférieur dans l'arborescence"))


        if "coop.agenda" in settings.INSTALLED_APPS:
            dated = generic.GenericRelation('coop_local.Dated')

        def label(self):
            return self.title

        # def can_publish_article(self, user):
        #     return (self.author == user)

        #def can_edit_article(self, user):
        #   return True
        #   test on URI, not on django user

        class Meta:
            verbose_name = _(u"article")
            verbose_name_plural = _(u"articles")
            abstract = True
            app_label = 'coop_local'


        # RDF stuff
        def isOpenData(self):
            return not self.isSection

        rdf_type = settings.NS.dcmi.Text
        base_mapping = [
            ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.title, 'title'), 'single_reverse'),
            ('single_mapping', (rdflib.URIRef(str(settings.NS['dct']) + 'abstract'), 'summary'), 'single_reverse'),  # bug in rdflib
            ('single_mapping', (settings.NS.dct.description, 'content'), 'single_reverse'),
            # ('single_mapping', (settings.NS.dce.type, 'rdf_type'), ''),  # DC compliance

            ('local_or_remote_mapping', (settings.NS.dct.creator, 'person'), 'local_or_remote_reverse'),
            ('local_or_remote_mapping', (settings.NS.dct.publisher, 'organization'), 'local_or_remote_reverse'),

            ('multi_mapping', (settings.NS.dct.subject, 'tags'), 'multi_reverse'),
            ]







# ----------- Idem for coop-cms NavTree            

    class CoopNavTree(BaseNavTree, URIModel):

        class Meta:
            verbose_name = _(u'Navigation tree')
            verbose_name_plural = _(u'Navigation trees')
            abstract = True
            app_label = 'coop_local'

        # RDF stuffs

        # As the "default" NavTree should NOT be exort as rdf data
        # A Nice solution is overwritte toRdfGraph method
        def isOpenData(self):
            return not (self.name == 'default' or self.name == 'spip')
 
        rdf_type = settings.NS.skos.ConceptScheme
        base_mapping = [
            ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
            ('single_mapping', (settings.NS.rdfs.label, 'name'), 'single_reverse'),
        ]


    






# -*- coding:utf-8 -*-

import floppyforms as forms
from coop_cms.forms import ArticleForm as CmsArticleForm
from coop_cms.settings import get_article_class
from djaloha.widgets import AlohaInput

class ArticleForm(CmsArticleForm):
    class Meta:
        model = get_article_class()
        fields = ('title', 'summary', 'content', 'logo')
        widgets = {
            'title': AlohaInput(text_color_plugin=False),
            'summary': AlohaInput(text_color_plugin=False),
            'content': AlohaInput(text_color_plugin=False),
        }


# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.conf import settings
#from coop.webid import webiduri
import sys

# https://code.djangoproject.com/ticket/10405#comment:11
# from django.db.models.loading import cache as model_cache
# if not model_cache.loaded:
#     model_cache.get_models()

from coop_local.urls import urlpatterns

handler500 = 'coop.views.SentryHandler500'

urlpatterns += patterns('',
    
    url(r'^', include('scanredirect.urls')),

    #Testing webid urls
    # url(r'^accounts/webidauth', 'coop.webid.views.test_login',
    #     name="webidauth-login"),
    # url(r'^webid/', include('django_webid.provider.urls')),
    # url(r'^auth/', include('coop.webid.urls')),

    #XXX remove this first url. Just debuggin'
    # url(r'^testpeople/(?P<username>\S+)', 'coop.webid.views.people',
    #     name="webidprovider-webid_uri.test"),
    # This url is used for assigning the WebID URI for
    # a user if no callback is given in settings.
    # url(r'^people/(?P<username>\S+)$',
    #     webiduri.WebIDProfileView.as_view(),
    #     name="webidprovider-webid_uri"),

    url(r'^admin_tools/', include('admin_tools.urls')),
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^admin/password_reset/$', 'django.contrib.auth.views.password_reset', name='admin_password_reset'),
    url(r'^admin/password_reset/done/$', 'django.contrib.auth.views.password_reset_done'),
    url(r'^reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', 'django.contrib.auth.views.password_reset_confirm'),
    url(r'^reset/done/$', 'django.contrib.auth.views.password_reset_complete'),
    #url(r'^org/$', 'coop_local.views.org_list', name="org_list"),  # exemple de view django-coop surchargee
)


# URLS for all Classification models

# from django.db.models.loading import get_models
# from coop.org.models import BaseClassification
# from django.views.generic.detail import DetailView

# for model in [x for x in get_models() if BaseClassification in x.__mro__]:
#     urlpatterns += patterns('',
#         url(r'^%s/(?P<slug>[-_\w|\W]+)/$' % model._meta.object_name.lower(),
#             DetailView.as_view(
#                 model=model,
#                 template_name='org/%s-detail.html' % model._meta.object_name.lower()
#             ), name='%s-detail' % model._meta.object_name.lower()),
#     )


# for local testing
if settings.DEBUG:
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += patterns('',
        (r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),
    )


if 'coop_tag' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        (r'^', include('coop_tag.urls')),
    )

if 'coop.agenda' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        (r'^', include('coop.agenda.urls')),
    )

if 'coop.mailing' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        (r'^', include('coop.mailing.urls')),
    )

urlpatterns += patterns('',
    (r'^forms/', include('forms_builder.forms.urls')),
    (r'^id/', include('uriredirect.urls')),
    (r'^data/', include('coop.data_urls')),
    (r'^', include('coop_geo.urls', app_name='coop_geo')),
    (r'^djaloha/', include('djaloha.urls')),
    (r'^', include('coop.urls')),
    (r'^', include('coop_cms.urls')),
)

if settings.DEBUG:
    urlpatterns += patterns('',
    # url(r'^', include('debug_toolbar_htmltidy.urls'))
    # url(r'^', include('debug_toolbar_user_panel.urls')),
)



# -*- coding:utf-8 -*-

from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization
from django.template import RequestContext
from django.conf import settings

# def org_list(request):
#     context = {}
#     context['org_list'] = Organization.objects.filter(active=True).order_by('statut')
#     return render_to_response('org/org_list.html',context,RequestContext(request))


if 'haystack' in settings.INSTALLED_APPS:
    from haystack.views import SearchView
    from coop_local.models import Article, Exchange, Event

    class ModelSearchView(SearchView):
        def __name__(self):
            return "FacetedSearchView"

        def extra_context(self):
            extra = super(ModelSearchView, self).extra_context()

            if self.results == []:
                extra['org'] = self.form.search().models(Organization)
                extra['exchange'] = self.form.search().models(Exchange)
                extra['article'] = self.form.search().models(Article)
                extra['event'] = self.form.search().models(Event)
            else:
                extra['org'] = self.results.models(Organization)
                extra['exchange'] = self.results.models(Exchange)
                extra['article'] = self.results.models(Article)
                extra['event'] = self.results.models(Event)

            return extra



# -*- coding:utf-8 -*-

import sys
from coop_local.local_settings import DEBUG
from coop_local.settings import DEBUG_SETTINGS

if DEBUG or ('runserver' in sys.argv):
    import firepython
    firepython.__api_version__ = '1.2'  # avoid known issue
    MIDDLEWARE_CLASSES = DEBUG_SETTINGS['middleware'] + [
        'debug_toolbar.middleware.DebugToolbarMiddleware',
        #'firepython.middleware.FirePythonDjango',
        ]
    INSTALLED_APPS = DEBUG_SETTINGS['apps'] + [
        'debug_toolbar',
        #'debug_toolbar_htmltidy',
        ]
    DEBUG_TOOLBAR_PANELS = [
        'debug_toolbar.panels.version.VersionDebugPanel',
        'debug_toolbar.panels.timer.TimerDebugPanel',
        'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
        'debug_toolbar.panels.headers.HeaderDebugPanel',
        'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
        'debug_toolbar.panels.template.TemplateDebugPanel',
        'debug_toolbar.panels.sql.SQLDebugPanel',
        'debug_toolbar.panels.signals.SignalDebugPanel',
        'debug_toolbar.panels.logger.LoggingPanel',
        #'debug_toolbar_htmltidy.panels.HTMLTidyDebugPanel',
    ]
    DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}

else:
    INSTALLED_APPS = DEBUG_SETTINGS['apps'] + ['raven.contrib.django']
    MIDDLEWARE_CLASSES = DEBUG_SETTINGS['middleware'] + [
        'raven.contrib.django.middleware.Sentry404CatchMiddleware',
        ]
    DEBUG_SETTINGS['logging']['root'] = {
        'level': 'WARNING',
        'handlers': ['sentry'],
    }
    DEBUG_SETTINGS['logging']['handlers']['sentry'] = {
            'level': 'ERROR',
            'class': 'raven.contrib.django.handlers.SentryHandler',
       }
    DEBUG_SETTINGS['logging']['loggers']['raven'] = {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        }
    DEBUG_SETTINGS['logging']['loggers']['sentry.errors'] = {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        }
    LOGGING = DEBUG_SETTINGS['logging']



# -*- coding: utf-8 -*-
# from django import forms
# from coop_local.models import MailingList
# from coop.mailing import soap


# class MailingListAdminForm(forms.ModelForm):
    
#     def clean_name(self):
#         name = self.cleaned_data['name']
#         if soap.exists(name):
#             raise forms.ValidationError(u"La liste %s existe déjà" % name)
#         return self.cleaned_data['name']

#     class Meta:
#         model = MailingList



# -*- coding:utf-8 -*-
from django.contrib.auth.models import User
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.template.defaultfilters import slugify
from django.contrib.contenttypes import generic
from django.contrib.sites.models import Site
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from coop.models import URIModel
import rdflib

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.models import Location


class BasePersonCategory(models.Model):
    label = models.CharField(blank=True, max_length=100)
    slug = exfields.AutoSlugField(populate_from=('label'), overwrite=True)

    class Meta:
        abstract = True
        verbose_name = _(u'Person category')
        verbose_name_plural = _(u'Person categories')
        app_label = 'coop_local'

    def __unicode__(self):
        return self.label

from coop.org.models import DISPLAY


class BasePerson(URIModel):
    user = models.OneToOneField(User, blank=True, null=True, unique=True, verbose_name=_(u'django user'), editable=False)
    username = models.CharField(blank=True, max_length=100, unique=True)
    category = models.ManyToManyField('coop_local.PersonCategory', blank=True, null=True, verbose_name=_(u'category'))
    last_name = models.CharField(_(u'last name'), max_length=100)
    first_name = models.CharField(_(u'first name'), max_length=100, null=True, blank=True)
    contact = generic.GenericRelation('coop_local.Contact')
    email = models.EmailField(_(u'personal email'), blank=True, help_text=_(u'will not be displayed on the website'))
    email_sha1 = models.CharField(_(u'email checksum'), max_length=250, blank=True, null=True)
    notes = models.TextField(_(u'notes'), blank=True, null=True)

    structure = models.CharField(blank=True, max_length=100)


    if 'coop.mailing' in settings.INSTALLED_APPS:
        subs = generic.GenericRelation('coop_local.Subscription')

    if "coop_geo" in settings.INSTALLED_APPS:
        location = models.ForeignKey(Location, null=True, blank=True, verbose_name=_(u'location'))
        location_display = models.PositiveSmallIntegerField(_(u'Display'), choices=DISPLAY.CHOICES, default=DISPLAY.USERS)

    class Meta:
        abstract = True
        verbose_name = _(u'Person')
        verbose_name_plural = _(u'Persons')
        app_label = 'coop_local'

    def __unicode__(self):
        return unicode('%s %s' % (self.first_name, self.last_name))

    #@models.permalink
    def get_absolute_url(self):
        return reverse_lazy('coop.person.views.public_profile', kwargs={'uuid': self.uuid})

    def has_user_account(self):
        return (self.user != None)
    has_user_account.boolean = True
    has_user_account.short_description = _(u'django account')

    def has_role(self):
        return (self.engagements.count() > 0)
    has_role.boolean = True
    has_role.short_description = _(u'has organization')

    def label(self):
        return self.__unicode__()

    def engagements(self):
        eng = []
        for e in self.engagement_set.all():
            eng.append({'initiative': e.initiative, 'role': e.role})
        return eng

    def save(self, *args, **kwargs):
        if self.email != '':
            import hashlib
            m = hashlib.sha1()
            m.update(self.email)
            self.email_sha1 = m.hexdigest()

        # create username slug if not set
        if self.username == '':
            from coop_local.models import Person
            newname = slugify(self.first_name).replace('-', '_') + '.' + \
                        slugify(self.last_name).replace('-', '_')
            if Person.objects.filter(username=newname).exists():
                offset = Person.objects.filter(username=newname).count()
                newname = newname + '_' + str(offset + 1)
            self.username = newname

        # synchronize fields with django User model
        if self.user:
            chg = False
            for field in ('username', 'first_name', 'last_name', 'email'):
                if(getattr(self.user, field) != getattr(self, field)):
                    setattr(self.user, field, getattr(self, field))
                    chg = True
            if(chg):
                self.user.save()
        super(BasePerson, self).save(*args, **kwargs)


    # RDF stuf
    rdf_type = settings.NS.person.Person
    base_mapping = [
        ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
        ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.familyName, 'last_name'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.givenName, 'first_name'), 'single_reverse'),
        ('single_mapping', (settings.NS.foaf.mbox_sha1sum, 'email_sha1'), 'single_reverse'),
        ('single_mapping', (settings.NS.skos.note, 'notes'), 'single_reverse'),
 
        ('multi_mapping', (settings.NS.dct.subject, 'tags'), 'multi_reverse'),
        ('multi_mapping', (settings.NS.ess.hasContactMedium, 'contact'), 'multi_reverse'),

        ('name_mapping', (settings.NS.foaf.name, 'username'), 'name_mapping_reverse'),
        ('location_mapping', (settings.NS.locn.location, 'location'), 'location_mapping_reverse')

    ]

    def name_mapping(self, rdfPred, djF, lang=None):
        if (self.first_name == "" or self.first_name == None) and self.last_name == "":
            return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(self.username, lang))]
        else:
            return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(u"%s %s" % (self.first_name, self.last_name), lang))]

    def name_mapping_reverse(self, g, rdfP, djF, lang=None):
        pass

    def location_mapping(self, rdfPred, djF, lang=None):
        if hasattr(self, 'location_display') and self.location_display == DISPLAY.PUBLIC:
            if self.location:
                return[(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.URIRef(self.location.uri))]
        return []

    def location_mapping_reverse(self, g, rdfP, djF, lang=None):
        pass



# -*- coding:utf-8 -*-
from django.shortcuts import render_to_response, redirect, get_object_or_404
from coop_local.models import Organization, Person, Article
from django.template import RequestContext
from django.core.urlresolvers import reverse


def public_profile(request, uuid):
    context = {}
    person = get_object_or_404(Person, uuid=uuid)
    context['person'] = person
    context['articles'] = Article.objects.filter(person=person).order_by('-created')

    return render_to_response('person/public_profile.html', context, RequestContext(request))



# -*- coding:utf-8 -*-
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from extended_choices import Choices
from django.core.urlresolvers import reverse
from decimal import Decimal
from django.conf import settings
from coop.models import URIModel
from coop.utils.fields import MultiSelectField
from django.contrib.contenttypes import generic
import datetime
import rdflib

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.models import Area, Location


class BaseProduct(URIModel):
    title = models.CharField(_('title'), blank=True, max_length=250)
    slug = exfields.AutoSlugField(populate_from='title', overwrite=True)
    description = models.TextField(_(u'description'), blank=True)
    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                                        verbose_name='publisher', related_name='products')

    remote_organization_uri = models.URLField(_(u'publisher URI'), blank=True, max_length=200, editable=False)
    remote_organization_label = models.CharField(_('organization'), blank=True, max_length=250)

    def __unicode__(self):
        return self.title + u' (' + self.organization.__unicode__() + u')'

    class Meta:
        abstract = True
        ordering = ['-modified']
        verbose_name = _(u'Product')
        verbose_name_plural = _(u'Products')
        app_label = 'coop_local'

    # RDF stuff
    rdf_type = settings.NS.schema.Product
    base_mapping = [
            ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
            ('single_mapping', (settings.NS.schema.name, 'title'), 'single_reverse'),
            ('single_mapping', (settings.NS.schema.description, 'description'), 'single_reverse'),
 
            ('local_or_remote_mapping', (settings.NS.schema.manufacturer, 'organization'), 'local_or_remote_reverse'),
    ]



EWAY = Choices(
    ('OFFER',   1,  _(u"I'm offering")),
    ('NEED',    2,  _(u"I'm looking for"))
)

ETYPE = Choices(
    ('PROD',    1,  _(u'Product or Material')),
    ('SERVE',   2,  _(u'Service')),
    ('SKILL',   3,  _(u'Skill')),
    ('COOP',    4,  _(u'Partnership')),
    ('QA',      5,  _(u'Question')),
)


class BaseExchangeMethod(models.Model):  # this model will be initialized with a fixture
    label = models.CharField(_(u'label'), max_length=250)
    uri = models.CharField(_(u'URI'), blank=True, max_length=250)
    etypes = MultiSelectField(_(u'applicable to'), max_length=250, null=True, blank=True, choices=ETYPE.CHOICES)

    def applications(self):
        return u", ".join([ETYPE.CHOICES_DICT[int(e)].__unicode__() for e in self.etypes])
    applications.short_description = _(u'applications')

    def __unicode__(self):
        return self.label

    class Meta:
        abstract = True
        verbose_name = _(u'Exchange method')
        verbose_name_plural = _(u'Exchange methods')
        app_label = 'coop_local'



class BaseExchange(URIModel):
    title = models.CharField(_('title'), max_length=250)
    description = models.TextField(_(u'description'), blank=True)

    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                            verbose_name=_('publisher'), related_name='exchanges')
    person = models.ForeignKey('coop_local.Person', blank=True, null=True, verbose_name=_(u'person'))

    eway = models.PositiveSmallIntegerField(_(u'exchange way'), choices=EWAY.CHOICES, default=EWAY.OFFER)
    etype = models.PositiveSmallIntegerField(_(u'exchange type'), choices=ETYPE.CHOICES)

    permanent = models.BooleanField(_(u'permanent'), default=True)
    expiration = models.DateField(_(u'expiration'), blank=True, null=True)
    slug = exfields.AutoSlugField(populate_from='title', overwrite=True)
    products = models.ManyToManyField('coop_local.Product', verbose_name=_(u'linked products'))

    # Linking to remote objects
    remote_person_uri = models.URLField(_(u'remote person URI'), blank=True, max_length=255, editable=False)
    remote_person_label = models.CharField(_(u'remote person label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the person record is not available locally'))
    remote_organization_uri = models.URLField(_(u'remote organization URI'), blank=True, max_length=255, editable=False)
    remote_organization_label = models.CharField(_(u'remote organization label'),
                                                max_length=250, blank=True, null=True,
                                                help_text=_(u'fill this only if the organization record is not available locally'))

    methods = models.ManyToManyField('coop_local.ExchangeMethod', verbose_name=_(u'exchange methods'))

    if "coop.agenda" in settings.INSTALLED_APPS:
        dated = generic.GenericRelation('coop_local.Dated')

    # coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:

        location = models.ForeignKey(   Location,
                                        verbose_name=_(u'location'),
                                        null=True, blank=True, related_name='exchange_location',
                                        help_text=_("choose a location among your registered locations."))
        area = models.ForeignKey(Area,
                                 verbose_name=_(u'interest area'),
                                 null=True, blank=True, related_name='exchange_area',
                                 help_text=_("choose an area among your registered impact areas."))

    def __unicode__(self):
        return unicode(self.title)

    def get_absolute_url(self):
        return reverse('exchange_detail', args=[self.id])

    def label(self):
        return self.title

    #TODO assign the record to the person editing it (form public) and provide an A-C choice in admin

    class Meta:
        abstract = True
        ordering = ('-modified',)
        verbose_name = _(u'Exchange')
        verbose_name_plural = _(u'Exchanges')
        app_label = 'coop_local'

    # title field needs a special handling. checkDirectMap does not work
    # because two rdf property use the coop_local_organization.title field
    # Thus we have to decide which one to use
    def updateField_title(self, dbfieldname, graph):
            title = list(graph.objects(rdflib.term.URIRef(self.uri), settings.NS.rdfs.label))
            if len(title) == 1:
                self.title = title[0]
                print "For id %s update the field %s" % (self.id, dbfieldname)
            else:
                print "    The field %s cannot be updated." % dbfieldname

    # RDF stuff
    rdf_type = [settings.NS.ess.Exchange, settings.NS.gr.Offering]

    base_mapping = [
            ('single_mapping', (settings.NS.dct.created, 'created'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.modified, 'modified'), 'single_reverse'),
            ('single_mapping', (settings.NS.rdfs.label, 'title'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.title, 'title'), 'single_reverse'),
            ('single_mapping', (settings.NS.dct.description, 'description'), 'single_reverse'),
            ('single_mapping', (settings.NS.rdfs.label, 'title'), 'single_reverse'),
            ('single_mapping', (settings.NS.gr.availabilityEnd, 'expiration'), 'single_reverse'),

            ('etype_mapping', (settings.NS.ov.category, 'etype'), 'etype_reverse_mapping'),

            ('single_mapping', (settings.NS.gr.eligibleRegions, 'area'), 'single_reverse'),
            ('single_mapping', (settings.NS.locn.location, 'location'), 'single_reverse'),

            ('local_or_remote_mapping', (settings.NS.dct.creator, 'person'), 'local_or_remote_reverse'),
            ('local_or_remote_mapping', (settings.NS.dct.publisher, 'organization'), 'local_or_remote_reverse'),


            ('multi_mapping', (settings.NS.dct.subject, 'tags'), 'multi_reverse'),
            ('multi_mapping', (settings.NS.ess.hasMethod, 'methods'), 'multi_reverse'),

            ('permanent_mapping', (settings.NS.gr.availabilityEnd, 'permanent'), 'permanent_mapping_reverse')

        ]

    _infinity_date = datetime.datetime(year=2050, month=12, day=31)

    def permanent_mapping(self, rdfPred, djF, lang=None):
        if getattr(self, djF):
            return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(self._infinity_date))]
        else:
            return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(self.expiration))]

    def etype_mapping(self, rdfPred, djF, lang=None):
        return [(rdflib.term.URIRef(self.uri), rdfPred, rdflib.term.Literal(ETYPE.CHOICES_DICT[self.etype]))]

    def etype_reverse_mapping(self, g, rdfPred, djF, lang=None):
        value = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        setattr(self, djF, ETYPE.REVERTED_CHOICES_DICT[value])

    def permanent_mapping_reverse(self, g, rdfPred, djF, lang=None):
        value = list(g.objects(rdflib.term.URIRef(self.uri), rdfPred))
        if len(value) == 1:
            if value[0].toPython() == self._infinity_date:
                setattr(self, djF, True)
                setattr(self, 'expiration', None)
            else:
                setattr(self, djF, False)
                setattr(self, 'expiration', value[0].toPython())


class BaseTransaction(models.Model):
    origin = models.ForeignKey('coop_local.Exchange', related_name='origin', verbose_name=_(u'origin'))
    destination = models.ForeignKey('coop_local.Exchange', related_name='destination', verbose_name=_(u'destination'))
    origin_org = models.ForeignKey('coop_local.Organization', related_name='contrats_vente', verbose_name=_(u'vendor'), blank=True, null=True)
    destination_org = models.ForeignKey('coop_local.Organization', related_name='contrats_achat', verbose_name=_(u'buyer'), blank=True, null=True)
    title = models.CharField(_('title'), blank=True, max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uuid = exfields.UUIDField()  # nécessaire pour URI ?

    def __unicode__(self):
        return self.title

    class Meta:
        abstract = True
        verbose_name = _(u'Transaction')
        verbose_name_plural = _(u'Transactions')
        app_label = 'coop_local'


# MODALITIES = Choices(
#     ('GIFT',    1,  _(u'Gift')),
#     ('TROC',    2,  _(u'Free exchange')),
#     ('CURR',    3,  _(u'Monetary exchange')),
# )
# UNITS = Choices(
#     ('EURO',    1,  _(u'€')),
#     ('SELH',    2,  _(u'Hours')),
#     ('PEZ',     3,  _(u'PEZ')),
# )


# class BasePaymentModality(models.Model):
#     exchange = models.ForeignKey('coop_local.Exchange', verbose_name=_(u'exchange'), related_name='modalities')
#     modality = models.PositiveSmallIntegerField(_(u'exchange type'), blank=True,
#                                                 choices=MODALITIES.CHOICES,
#                                                 default=MODALITIES.CURR)
#     amount = models.DecimalField(_(u'amount'), max_digits=12, decimal_places=2, default=Decimal(0.00), blank=True)
#     unit = models.PositiveSmallIntegerField(_(u'unit'), blank=True, null=True, choices=UNITS.CHOICES)

#     def __unicode__(self):
#         if(self.modality in [1, 2]):
#             return unicode(MODALITIES.CHOICES_DICT[self.modality])
#         elif(self.modality == 3 and self.amount > 0 and not self.unit == None):
#             return unicode("%s %s" % (self.amount, unicode(UNITS.CHOICES_DICT[self.unit])))
#         elif(self.modality == 3):
#             return unicode(_(u'Price unknown'))

#     def save(self, *args, **kwargs):
#         if not self.modality == 3:
#             self.amount = Decimal(0.00)
#             self.unit = None
#         super(BasePaymentModality, self).save(*args, **kwargs)

#     class Meta:
#         abstract = True
#         verbose_name = _(u'Payment modality')
#         verbose_name_plural = _(u'Payment modalities')







# -*- coding:utf-8 -*-

from django.db import models
from preferences.models import Preferences
from django.utils.translation import ugettext_lazy as _


class BaseSitePrefs(Preferences):
    __module__ = 'preferences.models'
    main_organization = models.ForeignKey('coop_local.Organization', null=True, blank=True, verbose_name=u'Organisme editeur', related_name='main_org')

    class Meta:
        abstract = True
        verbose_name = _('Site Preferences')
        app_label = 'coop_local'



# -*- coding:utf-8 -*-

from django.conf.urls.defaults import *
#from django.views.generic import ListView
#from coop.org.views import ISDetailView
#from coop_local.models import Organization


urlpatterns = patterns('',

    # sympa test
    url(r'^sympa_remote_list/(?P<name>\w+)$', 'coop.mailing.views.list'),


)
